@@ -42,13 +42,15 @@
 arg_pat="$1"
 
 # the version of this script
-bm_version=3.12.0
+bm_version=3.20.0
 
 # CONFIGURABLE VARIABLES ---------------------------------------
 # change as needed to reflect your environment/workloads
 
 # the path to the qsv binary, change this if you're not using the prebuilt binaries
 # e.g. you compiled a tuned version of qsv with different features and/or CPU optimizations enabled
+# qsv_bin=../target/release/qsv
+# qsv_bin=../target/debug/qsv
 qsv_bin=qsv
 # the path to the qsv binary that we dogfood to run the benchmarks
 # we use several optional features when dogfooding qsv (apply, luau & to)
@@ -272,6 +274,12 @@ function cleanup_files {
   rm -f results/run_info_work.tsv
   rm -f results/entry.csv
   rm -r -f split_tempdir
+  rm -r -f split_tempdir_chunks
+  rm -r -f split_tempdir_kbs
+  rm -r -f split_tempdir_idx
+  rm -r -f split_tempdir_idx_j1
+  rm -r -f split_tempdir_chunks_idx
+  rm -r -f split_tempdir_chunks_idx_j1
   rm -f benchmark_work.*
   rm -r -f benchmark_work
   rm -f extsort_sorted.csv
@@ -414,8 +422,6 @@ function run {
 # appropriate column names.
 
 run apply_calcconv "$qsv_bin apply calcconv --formatstr \"{Unique Key} meters in miles\" --new-column new_col $data"
-run apply_datefmt "$qsv_bin apply datefmt \"Created Date\" $data"
-run apply_datefmt_multi "$qsv_bin apply datefmt \"Created Date,Closed Date,Due Date\" $data"
 run apply_dynfmt "$qsv_bin apply dynfmt --formatstr \"{Created Date} {Complaint Type} - {BBL} {City}\" --new-column new_col $data"
 run apply_emptyreplace "$qsv_bin" apply emptyreplace \"Bridge Highway Name\" --replacement Unspecified "$data"
 run apply_op_eudex "$qsv_bin apply operations lower,eudex Agency --comparand Queens --new-column Agency_queens_soundex $data"
@@ -429,14 +435,25 @@ run cat_rows_flexible "$qsv_bin" cat rows --flexible "$data" data_unsorted.csv
 run cat_rowskey "$qsv_bin" cat rowskey "$data" data_unsorted.csv
 run count "$qsv_bin" count "$data"
 run count_flexible "$qsv_bin" count --flexible "$data"
+run count_polars_lowmem "$qsv_bin" count --low-memory "$data"
+run count_no_polars "$qsv_bin" count --no-polars "$data"
 run --index count_index "$qsv_bin" count "$data"
 run count_width "$qsv_bin" count --width "$data"
 run --index count_width_index "$qsv_bin" count --width "$data"
+run datefmt "$qsv_bin datefmt \"Created Date\" $data"
+run datefmt_multi "$qsv_bin datefmt \"Created Date,Closed Date,Due Date\" $data"
+run datefmt_multi_select "$qsv_bin datefmt '/(?i) date$/' $data"
+run datefmt_formatstr_newcol "$qsv_bin datefmt --formatstr '%V' \"Created Date\" --new-column week_number $data"
 run dedup "$qsv_bin" dedup "$data"
 run dedup_sorted "$qsv_bin" dedup data_sorted.csv
 run diff "$qsv_bin" diff "$data" data_unsorted.csv
 run enum "$qsv_bin" enum "$data"
+run enum_uuid "$qsv_bin" enum --uuid "$data"
+run enum_constant "$qsv_bin" enum --constant "NYC" "$data"
+run enum_copy "$qsv_bin" enum --copy Agency "$data"
 run excel "$qsv_bin" excel benchmark_data.xlsx
+run excel_metadata "$qsv_bin" excel --metadata c benchmark_data.xlsx
+run excel_metadata_short "$qsv_bin" excel --metadata s benchmark_data.xlsx
 run exclude "$qsv_bin" exclude \'Incident Zip\' "$data" \'Incident Zip\' data_to_exclude.csv
 run --index exclude_index "$qsv_bin" exclude \'Incident Zip\' "$data" \'Incident Zip\' data_to_exclude.csv
 run exclude_casei "$qsv_bin" exclude --ignore-case \'Incident Zip\' "$data" \'Incident Zip\' data_to_exclude.csv
@@ -471,6 +488,7 @@ run join "$qsv_bin" join \'Community Board\' "$data" community_board communitybo
 run join_casei "$qsv_bin" join \'Community Board\' "$data" community_board --ignore-case communityboards.csv
 run joinp "$qsv_bin" joinp \'Community Board\' "$data" community_board communityboards.csv
 run jsonl "$qsv_bin" jsonl benchmark_data.jsonl
+run jsonl_j1 "$qsv_bin" jsonl -j 1 benchmark_data.jsonl
 run luau_filter "$qsv_bin" luau filter \"Location == \'\'\" "$data"
 run luau_filter_colidx "$qsv_bin" luau filter --colindex \"Location == \'\'\" "$data"
 run luau_filter_no_globals "$qsv_bin" luau filter --no-globals \"Location == \'\'\" "$data"
@@ -529,11 +547,18 @@ run sortcheck_sorted "$qsv_bin" sortcheck data_sorted.csv
 run sortcheck_unsorted "$qsv_bin" sortcheck data_unsorted.csv
 run sortcheck_unsorted_all "$qsv_bin" sortcheck --all data_unsorted.csv
 run split "$qsv_bin" split --size 50000 split_tempdir "$data"
-run --index split_index "$qsv_bin" split --size 50000 split_tempdir "$data"
-run --index split_index_j1 "$qsv_bin" split --size 50000 -j 1 split_tempdir "$data"
+run split_chunks "$qsv_bin" split --chunks 20 split_tempdir_chunks "$data"
+run split_kbsize "$qsv_bin" split --kb-size 10000 split_tempdir_kbs "$data"
+run --index split_index "$qsv_bin" split --size 50000 split_tempdir_idx "$data"
+run --index split_index_j1 "$qsv_bin" split --size 50000 -j 1 split_tempdir_idx_j1 "$data"
+run --index split_chunks_index "$qsv_bin" split --chunks 20 split_tempdir_chunks_idx "$data"
+run --index split_chunks_index_j1 "$qsv_bin" split --chunks 20 -j 1 split_tempdir_chunks_idx_j1
 run sqlp "$qsv_bin" sqlp "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_aggregations "$qsv_bin" sqlp "$data" -Q '"select Borough, count(*) from _t_1 where \"Complaint Type\"='\''Noise'\'' group by Borough"'
 run sqlp_format_arrow "$qsv_bin" sqlp --format arrow "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_format_avro "$qsv_bin" sqlp --format avro "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
 run sqlp_format_json "$qsv_bin" sqlp --format json "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
+run sqlp_format_jsonl "$qsv_bin" sqlp --format jsonl "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
 run sqlp_format_parquet "$qsv_bin" sqlp --format parquet "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
 run sqlp_format_parquet_statistics "$qsv_bin" sqlp --format parquet --statistics "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
 run sqlp_lowmemory "$qsv_bin" sqlp "$data" -Q --low-memory '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
@@ -563,6 +588,8 @@ run to_parquet "$qsv_bin" to parquet benchmark_work "$data"
 run to_datapackage "$qsv_bin" to datapackage benchmark_work.json "$data"
 run tojsonl "$qsv_bin" tojsonl "$data"
 run tojsonl_j1 "$qsv_bin" tojsonl -j 1 "$data"
+run tojsonl_trim "$qsv_bin" tojsonl --trim "$data"
+run tojsonl_trim_j1 "$qsv_bin" tojsonl --trim -j 1 "$data"
 run --index tojsonl_index "$qsv_bin" tojsonl "$data"
 run --index tojsonl_index_j1 "$qsv_bin" tojsonl --jobs 1 "$data"
 run transpose "$qsv_bin" transpose "$data"
