@@ -3,12 +3,10 @@ applydp is a slimmed-down version of apply specifically created for Datapusher+.
 It "applies" a series of transformation functions to given CSV column/s. This can be used to
 perform typical data-wrangling tasks and/or to harmonize some values, etc.
 
-It has four subcommands:
+It has three subcommands:
  1. operations*   - 18 string, format & regex operators.
  2. emptyreplace* - replace empty cells with <--replacement> string.
- 3. datefmt*      - Formats recognized date/s (19 formats recognized) to
-                    a specified date format using <--formatstr>.
- 4. dynfmt        - Dynamically constructs a new column from other columns using
+ 3. dynfmt        - Dynamically constructs a new column from other columns using
                     the <--formatstr> template.
     * subcommand is multi-column capable.
 
@@ -81,7 +79,6 @@ Replace ' and ' with ' & ' in the description field.
 
   $ qsv applydp replace description --comparand ' and ' --replacement ' & ' file.csv
 
-
 You can also use this subcommand command to make a copy of a column:
 
   $ qsv applydp operations copy col_to_copy -c col_copy file.csv
@@ -112,45 +109,6 @@ case insensitive with 'None'.
 
 $ qsv applydp emptyreplace --replacement None '/(?i)^observation/' file.csv
 
-DATEFMT (multi-column capable)
-Formats a recognized date column to a specified format using <--formatstr>. 
-See https://github.com/jqnatividad/belt/tree/main/dateparser#accepted-date-formats for
-recognized date formats.
-See https://docs.rs/chrono/latest/chrono/format/strftime/ for
-accepted date formats for --formatstr.
-Defaults to ISO 8601/RFC 3339 format when --formatstr is not specified.
-( "%Y-%m-%dT%H:%M:%S%z" - e.g. 2001-07-08T00:34:60.026490+09:30 )
-
-Examples:
-Format dates in Open Date column to ISO 8601/RFC 3339 format:
-
-  $ qsv applydp datefmt 'Open Date' file.csv
-
-Format multiple date columns in file.csv to ISO 8601/RFC 3339 format:
-
-  $ qsv applydp datefmt 'Open Date,Modified Date,Closed Date' file.csv
-
-Format all columns that end with "_date" case-insensitive in file.csv to
-ISO 8601/RFC 3339 format:
-
-  $ qsv applydp datefmt '\(?i)_date$\' file.csv
-
-Format dates in OpenDate column using '%Y-%m-%d' format:
-
-  $ qsv applydp datefmt OpenDate --formatstr '%Y-%m-%d' file.csv
-
-Format multiple date columns using '%Y-%m-%d' format:
-
-  $ qsv applydp datefmt OpenDate,CloseDate,ReopenDate --formatstr '%Y-%m-%d' file.csv
-
-Get the week number for OpenDate and store it in the week_number column:
-
-  $ qsv applydp dateformat OpenDate --formatstr '%V' --new-column week_number file.csv
-
-Get the day of the week for several date columns and store it in the corresponding weekday columns:
-
-  $ qsv applydp dateformat OpenDate,CloseDate --formatstr '%u' --rename Open_weekday,Close_weekday file.csv
-
 DYNFMT
 Dynamically constructs a new column from other columns using the <--formatstr> template.
 The template can contain arbitrary characters. To insert a column value, enclose the
@@ -168,20 +126,18 @@ Create a new column 'FullName' from 'FirstName', 'MI', and 'LastName' columns:
 
   $ qsv applydp dynfmt --formatstr 'Sir/Madam {FirstName} {MI}. {LastName}' -c FullName file.csv
 
-
 For more extensive examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_applydp.rs.
 
 Usage:
 qsv applydp operations <operations> [options] <column> [<input>]
 qsv applydp emptyreplace --replacement=<string> [options] <column> [<input>]
-qsv applydp datefmt [--formatstr=<string>] [options] <column> [<input>]
 qsv applydp dynfmt --formatstr=<string> [options] --new-column=<name> [<input>]
 qsv applydp --help
 
 apply arguments:
     <column>                    The column/s to apply the transformation to.
                                 Note that the <column> argument supports multiple columns
-                                for the operations, emptyreplace & datefmt subcommands.
+                                for the operations & emptyreplace subcommands.
                                 See 'qsv select --help' for the format details.
 
     OPERATIONS subcommand:
@@ -192,12 +148,6 @@ apply arguments:
         --replacement=<string>  The string to to use to replace empty values.
         <column>                The column/s to check for emptiness.
 
-    DATEFMT subcommand:
-        --formatstr=<string>    The date format to use for the datefmt operation.
-                                See DATEFMT section in the --formatstr option below
-                                for more details.
-        <column>                The date column/s to apply the datefmt operation to.
-
     DYNFMT subcommand:
         --formatstr=<string>    The template to use for the dynfmt operation.
                                 See DYNFMT example above for more details.
@@ -210,22 +160,12 @@ applydp options:
     -r, --rename <name>         New name for the transformed column.
     -C, --comparand=<string>    The string to compare against for replace & similarity operations.
     -R, --replacement=<string>  The string to use for the replace & emptyreplace operations.
-    --prefer-dmy                Prefer to parse dates in dmy format. Otherwise, use mdy format.
-                                Only used with the DATEFMT subcommand.
-    --keep-zero-time            If a formatted date ends with "T00:00:00+00:00", keep the time
-                                instead of removing it. Only used with the DATEFMT subcommand.
     -f, --formatstr=<string>    This option is used by several subcommands:
 
                                 OPERATIONS:
                                   round
                                     The number of decimal places to round to (default: 3)
 
-                                DATEFMT: The date format to use. For formats, see
-                                  https://docs.rs/chrono/latest/chrono/format/strftime/
-                                  Default to ISO 8601 / RFC 3339 date & time format -
-                                  "%Y-%m-%dT%H:%M:%S%z" (e.g. 2001-07-08T00:34:60.026490+09:30)
-                                  [default: %+]
-
                                 DYNFMT: the template to use to construct a new column.
 
     -j, --jobs <arg>            The number of jobs to run in parallel.
@@ -246,7 +186,6 @@ use std::{str::FromStr, sync::OnceLock};
 
 use dynfmt::Format;
 use log::debug;
-use qsv_dateparser::parse_with_preference;
 use rayon::{
     iter::{IndexedParallelIterator, ParallelIterator},
     prelude::IntoParallelRefIterator,
@@ -292,25 +231,22 @@ enum Operations {
 
 #[derive(Deserialize)]
 struct Args {
-    arg_column:          SelectColumns,
-    cmd_operations:      bool,
-    arg_operations:      String,
-    cmd_datefmt:         bool,
-    cmd_dynfmt:          bool,
-    cmd_emptyreplace:    bool,
-    arg_input:           Option<String>,
-    flag_rename:         Option<String>,
-    flag_comparand:      String,
-    flag_replacement:    String,
-    flag_prefer_dmy:     bool,
-    flag_keep_zero_time: bool,
-    flag_formatstr:      String,
-    flag_batch:          u32,
-    flag_jobs:           Option<usize>,
-    flag_new_column:     Option<String>,
-    flag_output:         Option<String>,
-    flag_no_headers:     bool,
-    flag_delimiter:      Option<Delimiter>,
+    arg_column:       SelectColumns,
+    cmd_operations:   bool,
+    arg_operations:   String,
+    cmd_dynfmt:       bool,
+    cmd_emptyreplace: bool,
+    arg_input:        Option<String>,
+    flag_rename:      Option<String>,
+    flag_comparand:   String,
+    flag_replacement: String,
+    flag_formatstr:   String,
+    flag_batch:       u32,
+    flag_jobs:        Option<usize>,
+    flag_new_column:  Option<String>,
+    flag_output:      Option<String>,
+    flag_no_headers:  bool,
+    flag_delimiter:   Option<Delimiter>,
 }
 
 static REGEX_REPLACE: OnceLock<Regex> = OnceLock::new();
@@ -333,6 +269,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let headers = rdr.byte_headers()?.clone();
     let sel = rconfig.selection(&headers)?;
+    // safety: we just checked that sel is not empty in the previous line
     let column_index = *sel.iter().next().unwrap();
 
     let mut headers = rdr.headers()?.clone();
@@ -365,6 +302,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         // first, get the fields used in the dynfmt template
         let formatstr_re: &'static Regex = crate::regex_oncelock!(r"\{(?P<key>\w+)?\}");
         for format_fields in formatstr_re.captures_iter(&args.flag_formatstr) {
+            // safety: we already checked that the regex match is valid
             dynfmt_fields.push(format_fields.name("key").unwrap().as_str());
         }
         // we sort the fields so we can do binary_search
@@ -388,7 +326,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     #[derive(PartialEq)]
     enum ApplydpSubCmd {
         Operations,
-        DateFmt,
         DynFmt,
         EmptyReplace,
     }
@@ -407,8 +344,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             Err(e) => return Err(e),
         }
         ApplydpSubCmd::Operations
-    } else if args.cmd_datefmt {
-        ApplydpSubCmd::DateFmt
     } else if args.cmd_dynfmt {
         ApplydpSubCmd::DynFmt
     } else if args.cmd_emptyreplace {
@@ -428,7 +363,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // we set it to empty string
     let flag_replacement = if applydp_cmd == ApplydpSubCmd::Operations
         && ops_vec.contains(&Operations::Regex_Replace)
-        && args.flag_replacement.to_lowercase() == NULL_VALUE
+        && args.flag_replacement.to_ascii_lowercase() == NULL_VALUE
     {
         String::new()
     } else {
@@ -438,8 +373,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let flag_formatstr = args.flag_formatstr;
     let flag_new_column = args.flag_new_column;
 
-    let prefer_dmy = args.flag_prefer_dmy || rconfig.get_dmy_preference();
-
     // amortize memory allocation by reusing record
     #[allow(unused_assignments)]
     let mut batch_record = csv::StringRecord::new();
@@ -460,7 +393,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             match rdr.read_record(&mut batch_record) {
                 Ok(has_data) => {
                     if has_data {
-                        batch.push(batch_record.clone());
+                        batch.push(std::mem::take(&mut batch_record));
                     } else {
                         // nothing else to add to batch
                         break;
@@ -514,31 +447,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                             }
                         }
                     },
-                    ApplydpSubCmd::DateFmt => {
-                        let mut cell = String::new();
-                        for col_index in sel.iter() {
-                            record[*col_index].clone_into(&mut cell);
-                            if !cell.is_empty() {
-                                let parsed_date = parse_with_preference(&cell, prefer_dmy);
-                                if let Ok(format_date) = parsed_date {
-                                    let formatted_date =
-                                        format_date.format(&flag_formatstr).to_string();
-                                    if !args.flag_keep_zero_time
-                                        && formatted_date.ends_with("T00:00:00+00:00")
-                                    {
-                                        cell = formatted_date[..10].to_string();
-                                    } else {
-                                        cell = formatted_date;
-                                    }
-                                }
-                            }
-                            if flag_new_column.is_some() {
-                                record.push_field(&cell);
-                            } else {
-                                record = replace_column_value(&record, *col_index, &cell);
-                            }
-                        }
-                    },
                     ApplydpSubCmd::DynFmt => {
                         let mut cell = record[column_index].to_owned();
                         if !cell.is_empty() {
@@ -734,11 +642,13 @@ fn applydp_operations(
                 *cell = cell.replace(comparand, replacement);
             },
             Operations::Regex_Replace => {
+                // safety: we set REGEX_REPLACE in validate_operations()
                 let regexreplace = REGEX_REPLACE.get().unwrap();
                 *cell = regexreplace.replace_all(cell, replacement).to_string();
             },
             Operations::Round => {
                 if let Ok(num) = cell.parse::<f64>() {
+                    // safety: we set ROUND_PLACES in validate_operations()
                     *cell = util::round_num(num, *ROUND_PLACES.get().unwrap());
                 }
             },
