@@ -56,17 +56,21 @@ Excel options:
                                Negative indices start from the end (-1 = last sheet). 
                                If the sheet cannot be found, qsv will read the first sheet.
                                [default: 0]
-    --metadata <c|j|J>         Outputs workbook metadata in CSV or JSON format:
-                                 index, sheet_name, headers, num_columns, num_rows, safe_headers,
-                                 safe_headers_count, unsafe_headers, unsafe_headers_count and
-                                 duplicate_headers_count.
+    --metadata <c|j|J|s>       Outputs workbook metadata in CSV or JSON format:
+                                 index, sheet_name, headers, type, visible, num_columns, num_rows,
+                                 safe_headers, safe_headers_count, unsafe_headers, unsafe_headers_count
+                                 and duplicate_headers_count.
                                headers is a list of the first row which is presumed to be the header row.
+                               type is the sheet type (WorkSheet, DialogSheet, MacroSheet, ChartSheet, Vba).
+                               visible is the sheet visibility (Visible, Hidden, VeryHidden).
                                num_rows includes all rows, including the first row.
                                safe_headers is a list of header with "safe"(database-ready) names.
                                unsafe_headers is a list of headers with "unsafe" names.
                                duplicate_headers_count is a count of duplicate header names.
 
                                In CSV(c) mode, the output is in CSV format.
+                               In short CSV(s) mode, the output is in CSV format with only the
+                               index, sheet_name, type and visible fields.
                                
                                In JSON(j) mode, the output is minified JSON.
                                In Pretty JSON(J) mode, the output is pretty-printed JSON.
@@ -80,9 +84,11 @@ Excel options:
                                Also removes embedded linebreaks.
     --date-format <format>     Optional date format to use when formatting dates.
                                See https://docs.rs/chrono/latest/chrono/format/strftime/index.html
-                               for the full list of supported formats.
+                               for the full list of supported format specifiers.
                                Note that if a date format is invalid, qsv will fall back and
                                return the date as if no date-format was specified.
+     --keep-zero-time          Keep the time part of a date-time field if it is 00:00:00.
+                               By default, qsv will remove the time part if it is 00:00:00.
      --range <range>           An Excel format range, like C:T or C3:T25, to extract to the CSV.
      -j, --jobs <arg>          The number of jobs to run in parallel.
                                When not set, the number of jobs is set to the number of CPUs detected.
@@ -97,7 +103,7 @@ Common options:
 
 use std::{cmp, fmt::Write, path::PathBuf};
 
-use calamine::{open_workbook_auto, DataType, Range, Reader, SheetType};
+use calamine::{open_workbook, Data, Error, Range, Reader, SheetType, Sheets};
 use indicatif::HumanCount;
 use log::info;
 use rayon::prelude::*;
@@ -110,22 +116,24 @@ use crate::{
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input:        String,
-    flag_sheet:       String,
-    flag_metadata:    String,
-    flag_flexible:    bool,
-    flag_trim:        bool,
-    flag_output:      Option<String>,
-    flag_delimiter:   Option<Delimiter>,
-    flag_quiet:       bool,
-    flag_date_format: Option<String>,
-    flag_range:       String,
-    flag_jobs:        Option<usize>,
+    arg_input:           String,
+    flag_sheet:          String,
+    flag_metadata:       String,
+    flag_flexible:       bool,
+    flag_trim:           bool,
+    flag_output:         Option<String>,
+    flag_delimiter:      Option<Delimiter>,
+    flag_quiet:          bool,
+    flag_date_format:    Option<String>,
+    flag_keep_zero_time: bool,
+    flag_range:          String,
+    flag_jobs:           Option<usize>,
 }
 
 #[derive(PartialEq)]
 enum MetadataMode {
     Csv,
+    ShortCsv,
     Json,
     PrettyJSON,
     None,
@@ -228,10 +236,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .extension()
         .and_then(std::ffi::OsStr::to_str)
         .unwrap_or_default()
-        .to_lowercase();
-    let ods_flag = match format.as_str() {
-        "xls" | "xlsx" | "xlsm" | "xlsb" => false,
-        "ods" => true,
+        .to_ascii_lowercase();
+
+    let requested_range = args.flag_range.to_lowercase();
+
+    let mut ods_flag = false;
+    let mut workbook = match (format).as_str() {
+        "xls" | "xla" => Sheets::Xls(open_workbook(path).map_err(Error::Xls)?),
+        "xlsx" | "xlsm" => Sheets::Xlsx(open_workbook(path).map_err(Error::Xlsx)?),
+        "xlsb" => Sheets::Xlsb(open_workbook(path).map_err(Error::Xlsb)?),
+        "ods" => {
+            ods_flag = true;
+            Sheets::Ods(open_workbook(path).map_err(Error::Ods)?)
+        },
         _ => {
             return fail_incorrectusage_clierror!(
                 "\"{format}\" not supported. The excel command only supports the following file \
@@ -240,15 +257,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         },
     };
 
-    let requested_range = args.flag_range.to_lowercase();
-
-    let mut workbook = open_workbook_auto(path)?;
-
     let sheet_names = workbook.sheet_names();
     if sheet_names.is_empty() {
-        if ods_flag {
-            return fail_clierror!("{path} may be password protected.");
-        };
         return fail!("No sheets found.");
     }
     let num_sheets = sheet_names.len();
@@ -261,15 +271,17 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // set Metadata Mode
     let first_letter = args.flag_metadata.chars().next().unwrap_or_default();
     let metadata_mode = match first_letter {
+        'n' | 'N' => MetadataMode::None,
         'c' | 'C' => MetadataMode::Csv,
+        's' | 'S' => MetadataMode::ShortCsv,
         'j' => MetadataMode::Json,
         'J' => MetadataMode::PrettyJSON,
-        'n' | 'N' => MetadataMode::None,
         _ => {
-            return fail_incorrectusage_clierror!("Invalid mode: {}", args.flag_metadata);
+            return fail_incorrectusage_clierror!("Invalid metadata mode: {}", args.flag_metadata);
         },
     };
 
+    // check if we're exporting workbook metadata only
     if metadata_mode != MetadataMode::None {
         let mut excelmetadata_struct = MetadataStruct {
             filename,
@@ -286,7 +298,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let sheet_vec = sheet_names;
 
         for (i, sheet_name) in sheet_vec.iter().enumerate() {
-            let range = if let Some(result) = workbook.worksheet_range_at(i) {
+            let range = if metadata_mode == MetadataMode::ShortCsv {
+                Range::empty()
+            } else if let Some(result) = workbook.worksheet_range_at(i) {
                 match result {
                     Ok(result) => result,
                     Err(e) => {
@@ -303,7 +317,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             };
 
             let (header_vec, num_columns, num_rows, safenames_vec, unsafeheaders_vec, dupe_count) =
-                if range.is_empty() {
+                if metadata_mode == MetadataMode::ShortCsv || range.is_empty() {
                     (vec![], 0_usize, 0_usize, vec![], vec![], 0_usize)
                 } else {
                     let (num_rows, num_columns) = range.get_size();
@@ -408,6 +422,26 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 }
                 wtr.flush()?;
             },
+            MetadataMode::ShortCsv => {
+                let mut metadata_fields = Vec::with_capacity(4);
+                metadata_fields.extend_from_slice(&["index", "sheet_name", "type", "visible"]);
+                metadata_record = csv::StringRecord::from(metadata_fields);
+
+                wtr.write_record(&metadata_record)?;
+
+                for sheetmetadata in excelmetadata_struct.sheet {
+                    let metadata_values = vec![
+                        sheetmetadata.index.to_string(),
+                        sheetmetadata.name,
+                        sheetmetadata.typ,
+                        sheetmetadata.visible,
+                    ];
+                    metadata_record = csv::StringRecord::from(metadata_values);
+
+                    wtr.write_record(&metadata_record)?;
+                }
+                wtr.flush()?;
+            },
             MetadataMode::Json => {
                 let Ok(json_result) = serde_json::to_string(&excelmetadata_struct) else {
                     return fail!("Cannot create JSON");
@@ -428,6 +462,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         return Ok(());
     }
 
+    // --------------------------------------------------------------------
+    // we're not exporting metadata, we're exporting the spreadsheet to CSV
+
     // convert sheet_names to lowercase so we can do a case-insensitive compare
     let lower_sheet_names: Vec<String> = sheet_names.iter().map(|s| s.to_lowercase()).collect();
 
@@ -477,7 +514,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         // as we process the option case insensitively
         // safety: it's safe to use index access here because lower_sheet_names is a lowercase copy
         // of sheet_names
-        sheet = sheet_names[idx].clone();
+        sheet.clone_from(&sheet_names[idx]);
         idx
     } else {
         return fail_clierror!("Cannot get sheet index for {sheet}");
@@ -525,23 +562,23 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut trimmed_record = csv::StringRecord::with_capacity(500, col_count);
     let mut col_name: String;
 
-    // get the first row as header
+    // process the first row as the header row
     info!("exporting sheet ({sheet})... processing first row as header...");
     let first_row = match rows_iter.next() {
         Some(first_row) => first_row,
-        None => &[DataType::Empty],
+        None => &[Data::Empty],
     };
     for cell in first_row {
         col_name = match *cell {
-            DataType::String(ref s) => s.to_string(),
-            DataType::Empty => String::new(),
-            DataType::Error(ref _e) => String::new(),
-            DataType::Int(ref i) => i.to_string(),
-            DataType::DateTime(ref f) | DataType::Float(ref f) => f.to_string(),
-            DataType::Bool(ref b) => b.to_string(),
-            DataType::DateTimeIso(ref dt) => dt.to_string(),
-            DataType::DurationIso(ref d) => d.to_string(),
-            DataType::Duration(ref d) => d.to_string(),
+            Data::String(ref s) => s.to_string(),
+            Data::Empty => String::new(),
+            Data::Error(ref _e) => String::new(),
+            Data::Int(ref i) => i.to_string(),
+            Data::Float(ref f) => f.to_string(),
+            Data::DateTime(ref exceldatetime) => exceldatetime.to_string(),
+            Data::Bool(ref b) => b.to_string(),
+            Data::DateTimeIso(ref dt) => dt.to_string(),
+            Data::DurationIso(ref d) => d.to_string(),
         };
         record.push_field(&col_name);
     }
@@ -562,15 +599,21 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     info!("header: {record:?}");
     wtr.write_record(&record)?;
 
+    let no_date_format: bool;
+    // TODO: Clippy lint is broken, remove allow once fixed.
+    // https://github.com/rust-lang/rust-clippy/issues/12580
+    #[allow(clippy::manual_unwrap_or_default)]
     let date_format = if let Some(df) = args.flag_date_format {
+        no_date_format = false;
         df
     } else {
+        no_date_format = true;
         String::new()
     };
 
     let mut rows = Vec::with_capacity(row_count);
 
-    // process rest of the rows
+    // queue rest of the rows for processing as data rows
     for row in rows_iter {
         rows.push(row);
     }
@@ -581,14 +624,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // set chunk_size to number of rows per core/thread
     let chunk_size = row_count.div_ceil(ncpus);
 
+    let keep_zero_time = args.flag_keep_zero_time;
+
     let processed_rows: Vec<Vec<csv::StringRecord>> = rows
         .par_chunks(chunk_size)
         .map(|chunk| {
             let mut record = csv::StringRecord::with_capacity(500, col_count);
             let mut trimmed_record = csv::StringRecord::with_capacity(500, col_count);
-            let mut cell_date_flag: bool = false;
-            let mut float_val = 0_f64;
-            let mut float_flag: bool = false;
+            let mut float_val;
             let mut work_date;
             let mut ryu_buffer = ryu::Buffer::new();
             let mut itoa_buffer = itoa::Buffer::new();
@@ -599,95 +642,82 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             for row in chunk {
                 for cell in *row {
                     match *cell {
-                        DataType::Empty => record.push_field(""),
-                        DataType::String(ref s) => record.push_field(s),
-                        DataType::Int(ref i) => record.push_field(itoa_buffer.format(*i)),
-                        DataType::Float(ref f) => {
+                        Data::Empty => record.push_field(""),
+                        Data::String(ref s) => record.push_field(s),
+                        Data::Int(ref i) => record.push_field(itoa_buffer.format(*i)),
+                        Data::Float(ref f) => {
                             float_val = *f;
-                            float_flag = true;
-                            cell_date_flag = false;
+                            // push the ryu-formatted float value if its
+                            // not an integer or the candidate
+                            // integer is too big or too small to be an i64
+                            #[allow(clippy::cast_precision_loss)]
+                            if float_val.fract().abs() > f64::EPSILON
+                                || float_val > i64::MAX as f64
+                                || float_val < i64::MIN as f64
+                            {
+                                record.push_field(ryu_buffer.format_finite(float_val));
+                            } else {
+                                // its an i64 integer. We can't use ryu to format it, because it
+                                // will be formatted as a
+                                // float (have a ".0"). So we use itoa.
+                                record.push_field(itoa_buffer.format(float_val as i64));
+                            }
                         },
-                        DataType::DateTime(ref f) => {
-                            float_val = *f;
-                            float_flag = true;
-                            cell_date_flag = true;
-                        },
-                        DataType::Error(ref e) => record.push_field(&format!("{e:?}")),
-                        DataType::Bool(ref b) => {
-                            record.push_field(if *b { "true" } else { "false" });
-                        },
-                        DataType::DateTimeIso(ref dt) => record.push_field(dt),
-                        DataType::DurationIso(ref d) => record.push_field(d),
-                        DataType::Duration(ref d) => record.push_field(ryu_buffer.format(*d)),
-                    };
-
-                    #[allow(clippy::cast_precision_loss)]
-                    if float_flag {
-                        if cell_date_flag {
-                            // its a date, so convert it
-                            work_date = if float_val.fract() > f64::EPSILON {
-                                // if it has a fractional part, then its a datetime
-                                if let Some(dt) = cell.as_datetime() {
-                                    if date_format.is_empty() {
+                        Data::DateTime(ref edt) => {
+                            if edt.is_datetime() {
+                                if let Some(dt) = edt.as_datetime() {
+                                    if no_date_format {
                                         // no date format specified, so we'll just use the
                                         // default format for the datetime
-                                        dt.to_string()
+                                        work_date = dt.to_string();
                                     } else {
                                         // a date format was specified, so we'll use it
-                                        (formatted_date).clear();
+                                        formatted_date.clear();
                                         if write!(formatted_date, "{}", dt.format(&date_format))
                                             .is_ok()
                                         {
                                             // the format string was ok, so use to_string()
                                             // to actually apply the DelayedFormat
-                                            formatted_date.to_string()
+                                            work_date = formatted_date.to_string();
                                         } else {
                                             // if there was a format error, revert to the
                                             // default format
-                                            dt.to_string()
+                                            work_date = dt.to_string();
                                         }
                                     }
-                                } else {
-                                    format!("ERROR: Cannot convert {float_val} to datetime")
-                                }
-                            } else if let Some(d) = cell.as_date() {
-                                // if it has no fractional part and calamine can return it
-                                // as_date, then its a date
-                                if date_format.is_empty() {
-                                    d.to_string()
-                                } else {
-                                    formatted_date.clear();
-                                    if write!(formatted_date, "{}", d.format(&date_format)).is_ok()
-                                    {
-                                        formatted_date.to_string()
-                                    } else {
-                                        d.to_string()
+                                    if !keep_zero_time && work_date.ends_with(" 00:00:00") {
+                                        work_date.truncate(work_date.len() - 9);
                                     }
+                                } else {
+                                    // if the datetime is invalid, just return the datetime as a
+                                    // string this should never
+                                    // happen as we did a is_datetime check
+                                    // before we got here. We're just doing it so that work_date
+                                    // is initialized properly without wasting an allocation
+                                    work_date = edt.to_string();
                                 }
                             } else {
-                                format!("ERROR: Cannot convert {float_val} to date")
+                                // its not a datetime, its a duration
+                                // return the duration as a string in ISO 8601 format
+                                // https://www.digi.com/resources/documentation/digidocs/90001488-13/reference/r_iso_8601_duration_format.htm
+                                // safety: we know this is a duration coz we did a is_datetime check
+                                // above & ExcelDataTime only has 2 variants, DateTime & Duration
+                                work_date = edt.as_duration().unwrap().to_string();
                             };
+
                             record.push_field(&work_date);
-                        // its not a date, so just push the ryu-formatted float value if its
-                        // not an integer or the candidate
-                        // integer is too big or too small to be an i64
-                        } else if float_val.fract().abs() > f64::EPSILON
-                            || float_val > i64::MAX as f64
-                            || float_val < i64::MIN as f64
-                        {
-                            record.push_field(ryu_buffer.format_finite(float_val));
-                        } else {
-                            // its an i64 integer. We can't use ryu to format it, because it
-                            // will be formatted as a
-                            // float (have a ".0"). So we use itoa.
-                            record.push_field(itoa_buffer.format(float_val as i64));
-                        }
-                        // reset the float flag
-                        float_flag = false;
-                    }
+                        },
+                        Data::Error(ref e) => record.push_field(&format!("{e}")),
+                        Data::Bool(ref b) => {
+                            record.push_field(if *b { "true" } else { "false" });
+                        },
+                        Data::DateTimeIso(ref dt) => record.push_field(dt),
+                        Data::DurationIso(ref d) => record.push_field(d),
+                    };
                 }
 
                 if trim {
+                    // record.trim() is faster than trimming each field piecemeal
                     record.trim();
                     record.iter().for_each(|field| {
                         if field.contains('\n') {
@@ -700,8 +730,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     trimmed_record.clear();
                 }
 
-                processed_chunk.push(record.clone());
-                record.clear();
+                // we use mem::take here to avoid a clone/allocation of the record
+                // it also has the nice side-effect of clearing the record, so we don't
+                // need to call clear() on it.
+                processed_chunk.push(std::mem::take(&mut record));
             }
             processed_chunk
         })
