@@ -383,7 +383,7 @@ use rayon::{
 use regex::Regex;
 use serde::Deserialize;
 use serde_json::json;
-use simple_home_dir::expand_tilde;
+use simple_expand_tilde::expand_tilde;
 use tempfile::tempdir;
 use url::Url;
 use uuid::Uuid;
@@ -1117,7 +1117,7 @@ async fn geocode_main(args: Args) -> CliResult<()> {
             match rdr.read_record(&mut batch_record) {
                 Ok(has_data) => {
                     if has_data {
-                        batch.push(batch_record.clone());
+                        batch.push(std::mem::take(&mut batch_record));
                     } else {
                         // nothing else to add to batch
                         break;
@@ -1215,7 +1215,7 @@ async fn geocode_main(args: Args) -> CliResult<()> {
                         if !invalid_result.is_empty() {
                             // --invalid-result is set, so use that instead
                             // otherwise, we leave cell untouched.
-                            cell = invalid_result.clone();
+                            cell.clone_from(&invalid_result);
                         }
                     }
                 }
