@@ -4,7 +4,7 @@ Joins two sets of CSV data on the specified columns using the Pola.rs engine.
 The default join operation is an 'inner' join. This corresponds to the
 intersection of rows on the keys specified.
 
-Unlike the join command, joinp can process files larger than RAM, is multi-threaded,
+Unlike the join command, joinp can process files larger than RAM, is multithreaded,
 has join key validation, pre-join filtering, supports asof joins & its output doesn't
 have duplicate columns.
 
@@ -162,7 +162,7 @@ joinp options:
 Common options:
     -h, --help             Display this message
     -o, --output <file>    Write output to <file> instead of stdout.
-    -d, --delimiter <arg>  The field delimiter for reading CSV data.
+    -d, --delimiter <arg>  The field delimiter for reading/writing CSV data.
                            Must be a single character. (default: ,)
     -Q, --quiet            Do not return join shape to stderr.
 "#;
@@ -184,10 +184,9 @@ use polars::{
     sql::SQLContext,
 };
 use serde::Deserialize;
-use smartstring;
 use tempfile::tempdir;
 
-use crate::{cmd::sqlp::compress_output_if_needed, config::Delimiter, util, CliError, CliResult};
+use crate::{cmd::sqlp::compress_output_if_needed, config::Delimiter, util, CliResult};
 
 #[derive(Deserialize)]
 struct Args {
@@ -226,12 +225,6 @@ struct Args {
     flag_quiet:            bool,
 }
 
-impl From<polars::error::PolarsError> for CliError {
-    fn from(err: polars::error::PolarsError) -> CliError {
-        CliError::Other(format!("Polars error: {err:?}"))
-    }
-}
-
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut args: Args = util::get_args(USAGE, argv)?;
 
@@ -398,6 +391,7 @@ impl JoinStruct {
                 streaming:           self.streaming,
                 fast_projection:     true,
                 eager:               false,
+                row_estimate:        true,
             }
         };
         log::debug!("Optimization state: {optimization_state:?}");
@@ -441,9 +435,12 @@ impl JoinStruct {
             join_results
         };
 
-        // no need to use buffered writer here, as CsvWriter already does that
+        let mut out_delim = self.delim;
         let mut out_writer = match self.output {
             Some(ref output_file) => {
+                out_delim = tsvtab_delim(output_file, self.delim);
+
+                // no need to use buffered writer here, as CsvWriter already does that
                 let path = Path::new(&output_file);
                 Box::new(File::create(path).unwrap()) as Box<dyn Write>
             },
@@ -455,7 +452,7 @@ impl JoinStruct {
 
         CsvWriter::new(&mut out_writer)
             .include_header(true)
-            .with_separator(self.delim)
+            .with_separator(out_delim)
             .with_datetime_format(self.datetime_format)
             .with_date_format(self.date_format)
             .with_time_format(self.time_format)
@@ -520,7 +517,7 @@ impl Args {
                 .has_header(true)
                 .with_missing_is_null(self.flag_nulls)
                 .with_comment_prefix(comment_char.as_deref())
-                .with_separator(delim)
+                .with_separator(tsvtab_delim(&self.arg_input1, delim))
                 .with_infer_schema_length(num_rows)
                 .with_try_parse_dates(try_parsedates)
                 .low_memory(low_memory)
@@ -545,7 +542,7 @@ impl Args {
                 .has_header(true)
                 .with_missing_is_null(self.flag_nulls)
                 .with_comment_prefix(comment_char.as_deref())
-                .with_separator(delim)
+                .with_separator(tsvtab_delim(&self.arg_input2, delim))
                 .with_infer_schema_length(num_rows)
                 .with_try_parse_dates(try_parsedates)
                 .low_memory(low_memory)
@@ -580,3 +577,21 @@ impl Args {
         })
     }
 }
+
+/// if the file has a TSV or TAB extension, we automatically use tab as the delimiter
+/// otherwise, we use the delimiter specified by the user
+pub fn tsvtab_delim<P: AsRef<Path>>(file: P, orig_delim: u8) -> u8 {
+    let inputfile_extension = file
+        .as_ref()
+        .extension()
+        .and_then(std::ffi::OsStr::to_str)
+        .unwrap_or_default();
+
+    if inputfile_extension.eq_ignore_ascii_case("tsv")
+        || inputfile_extension.eq_ignore_ascii_case("tab")
+    {
+        b'\t'
+    } else {
+        orig_delim
+    }
+}
