@@ -1,6 +1,6 @@
 static USAGE: &str = r#"
 Create multiple new computed columns, filter rows or compute aggregations by 
-executing a Luau 0.606 script for every row (SEQUENTIAL MODE) or for
+executing a Luau 0.616 script for every row (SEQUENTIAL MODE) or for
 specified rows (RANDOM ACCESS MODE) of a CSV file.
 
 Luau is not just another qsv command. It is qsv's Domain-Specific Language (DSL)
@@ -246,9 +246,8 @@ use indicatif::{ProgressBar, ProgressDrawTarget, ProgressStyle};
 use log::{debug, info, log_enabled};
 use mlua::{Lua, LuaSerdeExt, Value};
 use serde::Deserialize;
-use simple_home_dir::expand_tilde;
+use simple_expand_tilde::expand_tilde;
 use strum_macros::IntoStaticStr;
-use tempfile;
 
 use crate::{
     config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
@@ -1413,7 +1412,7 @@ fn create_index(arg_input: &Option<String>) -> Result<bool, CliError> {
         return Ok(false);
     };
 
-    if input.to_lowercase().ends_with(".sz") {
+    if input.to_ascii_lowercase().ends_with(".sz") {
         log::warn!("qsv_autoindex() does not work with snappy files.");
         return Ok(false);
     }
@@ -1481,7 +1480,7 @@ fn setup_helpers(
             let val = luau.from_value::<serde_json::Value>(val)?;
             let val_str = &serde_json::to_string_pretty(&val).unwrap_or_default();
             if idx == 0 {
-                log_level = val_str.trim_matches('"').to_lowercase();
+                log_level = val_str.trim_matches('"').to_ascii_lowercase();
             } else {
                 log_msg.push_str(val_str.trim_matches('"'));
                 if idx == u8::MAX {
@@ -1918,7 +1917,7 @@ fn setup_helpers(
                 if log_enabled!(log::Level::Debug) {
                     log::debug!("qsv command stdout: {child_stdout} stderr: {child_stderr}");
                 } else {
-                    log::info!("qsv command executed.");
+                    log::info!("qsv command executed: {qsv_args}");
                 }
 
                 let output_table = luau.create_table()?;
@@ -1955,16 +1954,21 @@ fn setup_helpers(
         enum ShellCmd {
             Awk,
             Cat,
+            Ckan,
+            Ckanapi,
             Cp,
             Cut,
             Df,
             Echo,
             Grep,
             Head,
+            Jq,
+            Jql,
             Ls,
             Mkdir,
             Mv,
             Nl,
+            Psql,
             Pwd,
             Rg,
             Sed,
@@ -1983,8 +1987,8 @@ fn setup_helpers(
             return helper_err!(
                 "qsv_shellcmd",
                 "Invalid shell command: \"{shellcmd}\". Only the following commands are allowed: \
-                 awk, cat, cp, cut, df, echo, rg, grep, head, ls, mkdir, mv, nl, pwd, sed, sort, \
-                 tail, touch, tr, uname, uniq, wc, whoami"
+                 awk, cat, ckan, ckanapi, cp, cut, df, echo, rg, grep, head, jq, jql, ls, mkdir, \
+                 mv, nl, psql, pwd, sed, sort, tail, touch, tr, uname, uniq, wc, whoami"
             );
         };
 
