@@ -7,6 +7,7 @@ pub mod behead;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod cat;
 pub mod count;
+pub mod datefmt;
 pub mod dedup;
 pub mod describegpt;
 pub mod diff;
@@ -41,7 +42,7 @@ pub mod index;
 pub mod input;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod join;
-#[cfg(all(feature = "polars", feature = "feature_capable"))]
+#[cfg(feature = "polars")]
 pub mod joinp;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod jsonl;
@@ -54,7 +55,6 @@ pub mod pseudo;
 pub mod python;
 pub mod rename;
 pub mod replace;
-#[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod reverse;
 pub mod safenames;
 pub mod sample;
@@ -70,7 +70,7 @@ pub mod sort;
 pub mod sortcheck;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod split;
-#[cfg(all(feature = "polars", feature = "feature_capable"))]
+#[cfg(feature = "polars")]
 pub mod sqlp;
 pub mod stats;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
