@@ -8,6 +8,10 @@ field names & data types) using a Viterbi algorithm. (https://en.wikipedia.org/w
 last modified date. If --no-infer is enabled, it doesn't even bother to infer the CSV's schema.
 This makes it useful for accelerated CKAN harvesting and for checking stale/broken resource URLs.
 
+It detects more than 120 mime types, including CSV, MS Office/Open Document files, JSON, XML, 
+PDF, PNG, JPEG and specialized geospatial formats like GPX, GML, KML, TML, TMX, TSX, TTML.
+see https://docs.rs/file-format/latest/file_format/#reader-features
+
 NOTE: This command "sniffs" a CSV's schema by sampling the first n rows (default: 1000)
 of a file. Its inferences are sometimes wrong if the the file is too small to infer a pattern
 or if the CSV has unusual formatting - with atypical delimiters, quotes, etc.
@@ -80,6 +84,10 @@ sniff options:
                              (Unsigned, Signed => Integer, Text => String, everything else the same)
     --no-infer               Do not infer the schema. Only return the file's mime type, size and
                              last modified date. Use this to use sniff as a general mime type detector.
+                             Note that CSV and TSV files will only be detected as mime type plain/text
+                             in this mode.
+    --just-mime              Only return the file's mime type. Use this to use sniff as a general
+                             mime type detector. Synonym for --no-infer.
     --quick                  When sniffing a non-CSV remote file, only download the first chunk of the file
                              before attempting to detect the mime type. This is faster but less accurate as
                              some mime types cannot be detected with just the first downloaded chunk.
@@ -97,27 +105,29 @@ use std::{
     fmt, fs,
     io::{copy, Seek, SeekFrom, Write},
     path::PathBuf,
-    time::{Duration, SystemTime},
+    time::Duration,
 };
 
 use bytes::Bytes;
 use file_format::FileFormat;
 use futures::executor::block_on;
 use futures_util::StreamExt;
+use indicatif::HumanCount;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
-use indicatif::{HumanBytes, HumanCount, ProgressBar, ProgressDrawTarget, ProgressStyle};
+use indicatif::{HumanBytes, ProgressBar, ProgressDrawTarget, ProgressStyle};
 use qsv_sniffer::{DatePreference, SampleSize, Sniffer};
 use reqwest::Client;
 use serde::{Deserialize, Serialize};
 use serde_json::json;
 use tabwriter::TabWriter;
 use tempfile::NamedTempFile;
-use thousands::Separable;
 use url::Url;
 
 use crate::{
     config::{Config, Delimiter},
-    util, CliResult,
+    util,
+    util::format_systemtime,
+    CliResult,
 };
 
 #[derive(Deserialize)]
@@ -135,6 +145,7 @@ struct Args {
     flag_user_agent:     Option<String>,
     flag_stats_types:    bool,
     flag_no_infer:       bool,
+    flag_just_mime:      bool,
     flag_quick:          bool,
     flag_harvest_mode:   bool,
 }
@@ -198,7 +209,7 @@ impl fmt::Display for SniffStruct {
         writeln!(
             f,
             "Preamble Rows: {}",
-            self.preamble_rows.separate_with_commas()
+            HumanCount(self.preamble_rows as u64)
         )?;
         writeln!(f, "Quote Char: {}", self.quote_char)?;
         writeln!(f, "Flexible: {}", self.flexible)?;
@@ -208,30 +219,26 @@ impl fmt::Display for SniffStruct {
         writeln!(
             f,
             "Retrieved Size (bytes): {}",
-            self.retrieved_size.separate_with_commas()
+            HumanCount(self.retrieved_size as u64)
         )?;
         writeln!(
             f,
             "File Size (bytes): {}",
-            self.file_size.separate_with_commas()
+            HumanCount(self.file_size as u64)
         )?;
         writeln!(
             f,
             "Sampled Records: {}",
-            self.sampled_records.separate_with_commas()
+            HumanCount(self.sampled_records as u64)
         )?;
         writeln!(f, "Estimated: {}", self.estimated)?;
-        writeln!(
-            f,
-            "Num Records: {}",
-            self.num_records.separate_with_commas()
-        )?;
+        writeln!(f, "Num Records: {}", HumanCount(self.num_records as u64))?;
         writeln!(
             f,
             "Avg Record Len (bytes): {}",
-            self.avg_record_len.separate_with_commas()
+            HumanCount(self.avg_record_len as u64)
         )?;
-        writeln!(f, "Num Fields: {}", self.num_fields.separate_with_commas())?;
+        writeln!(f, "Num Fields: {}", HumanCount(self.num_fields as u64))?;
         writeln!(f, "Stats Types: {}", self.stats_types)?;
         writeln!(f, "Fields:")?;
 
@@ -334,8 +341,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                         util::timeout_secs(args.flag_timeout).unwrap_or(30),
                     ))
                     .send()
-                    .await
-                    .map_err(|e| format!("Download failed: {e}"))?;
+                    .await?;
 
                 let last_modified = match res.headers().get("Last-Modified") {
                     Some(lm) => match lm.to_str() {
@@ -416,6 +422,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                 let mut file = NamedTempFile::new()?;
                 let mut downloaded = 0_usize;
                 let mut stream = res.bytes_stream();
+                #[cfg(any(feature = "feature_capable", feature = "lite"))]
                 let mut downloaded_lines = 0_usize;
                 #[allow(unused_assignments)]
                 let mut chunk = Bytes::new(); // amortize the allocation
@@ -423,10 +430,9 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
 
                 // download chunks until we have the desired sample size
                 while let Some(item) = stream.next().await {
-                    chunk = item.map_err(|_| "Error while downloading file")?;
+                    chunk = item?;
 
-                    file.write_all(&chunk)
-                        .map_err(|_| "Error while writing to file")?;
+                    file.write_all(&chunk)?;
                     let chunk_len = chunk.len();
 
                     if downloaded == 0 && !snappy_flag && args.flag_quick {
@@ -447,6 +453,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     }
 
                     // check if we're downloading the entire file
+                    #[cfg(any(feature = "feature_capable", feature = "lite"))]
                     if lines_sample_size != usize::MAX {
                         // we're not downloading the entire file, so we need to
                         // scan chunk for newlines
@@ -607,22 +614,13 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     },
                 }
 
-                let metadata = fs::metadata(&path)
-                    .map_err(|_| format!("Cannot get metadata for file '{path}'"))?;
+                let metadata = fs::metadata(&path)?;
 
                 let file_size = metadata.len() as usize;
                 let last_modified = match metadata.modified() {
                     Ok(time) => {
-                        let timestamp = time
-                            .duration_since(SystemTime::UNIX_EPOCH)
-                            .unwrap()
-                            .as_secs();
-                        let naive = chrono::NaiveDateTime::from_timestamp_opt(timestamp as i64, 0)
-                            .unwrap_or_default();
-                        let datetime =
-                            chrono::DateTime::<chrono::Utc>::from_utc(naive, chrono::Utc);
                         // format the datetime to RFC3339
-                        format!("{datetime}", datetime = datetime.format("%+"))
+                        format_systemtime(time, "%+")
                     },
                     Err(_) => "N/A".to_string(),
                 };
@@ -657,9 +655,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
             return fail_clierror!("stdin input is not UTF8-encoded");
         }
 
-        let metadata = file
-            .metadata()
-            .map_err(|_| "Cannot get metadata for stdin file")?;
+        let metadata = file.metadata()?;
 
         let file_size = metadata.len() as usize;
         // set last_modified to now in RFC3339 format
@@ -713,9 +709,14 @@ async fn sniff_main(mut args: Args) -> CliResult<()> {
             Some("CKAN-harvest/$QSV_VERSION ($QSV_TARGET; $QSV_BIN_NAME)".to_string());
     }
 
+    if args.flag_just_mime {
+        args.flag_no_infer = true;
+    }
+
     let mut sample_size = args.flag_sample;
+    let flag_json = args.flag_json || args.flag_pretty_json;
     if sample_size < 0.0 {
-        if args.flag_json || args.flag_pretty_json {
+        if flag_json {
             let json_result = json!({
                 "errors": [{
                     "title": "sniff error",
@@ -724,7 +725,7 @@ async fn sniff_main(mut args: Args) -> CliResult<()> {
             });
             return fail_clierror!("{json_result}");
         }
-        return fail_clierror!("Sample size must be greater than or equal to zero.");
+        return fail_incorrectusage_clierror!("Sample size must be greater than or equal to zero.");
     }
 
     let sniffed_ts = chrono::Utc::now().to_rfc3339();
@@ -760,7 +761,7 @@ async fn sniff_main(mut args: Args) -> CliResult<()> {
         };
         let last_modified = sfile_info.last_modified;
 
-        if args.flag_json || args.flag_pretty_json {
+        if flag_json {
             if args.flag_no_infer {
                 let json_result = json!({
                     "title": "sniff mime type",
@@ -817,7 +818,7 @@ async fn sniff_main(mut args: Args) -> CliResult<()> {
             Err(e) => {
                 cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
 
-                if args.flag_json || args.flag_pretty_json {
+                if flag_json {
                     let json_result = json!({
                         "errors": [{
                             "title": "count rows error",
@@ -840,7 +841,7 @@ async fn sniff_main(mut args: Args) -> CliResult<()> {
     if n_rows == 0 {
         cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
 
-        if args.flag_json || args.flag_pretty_json {
+        if flag_json {
             let json_result = json!({
                 "errors": [{
                     "title": "sniff error",
@@ -991,7 +992,7 @@ async fn sniff_main(mut args: Args) -> CliResult<()> {
     cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
 
     // safety: we just created all these json values above, so they are safe to unwrap
-    if args.flag_json || args.flag_pretty_json {
+    if flag_json {
         if sniff_error.is_none() {
             if args.flag_pretty_json {
                 println!(
