@@ -1,101 +1,111 @@
 static USAGE: &str = r#"
 Run blazing-fast Polars SQL queries against several CSVs - replete with joins, aggregations,
-grouping, sorting, and more - working on larger than memory CSV files.
+grouping, table functions, sorting, and more - working on larger than memory CSV files.
 
 Polars SQL is a SQL dialect, converting SQL queries to fast Polars LazyFrame expressions.
 
 For a list of SQL functions and keywords supported by Polars SQL, see
-https://github.com/pola-rs/polars/blob/rs-0.36.2/crates/polars-sql/src/functions.rs
-https://github.com/pola-rs/polars/blob/rs-0.36.2/crates/polars-sql/src/keywords.rs and
+https://github.com/pola-rs/polars/blob/rs-0.38.3/crates/polars-sql/src/functions.rs
+https://github.com/pola-rs/polars/blob/rs-0.38.3/crates/polars-sql/src/keywords.rs and
 https://github.com/pola-rs/polars/issues/7227
 
 Returns the shape of the query result (number of rows, number of columns) to stderr.
 
 Example queries:
 
-  qsv sqlp data.csv 'select * from data where col1 > 10 order by col2 desc limit 20'
+   qsv sqlp data.csv 'select * from data where col1 > 10 order by col2 desc limit 20'
 
-  qsv sqlp data.csv 'select col1, col2 as friendlyname from data' --format parquet --output data.parquet
+   qsv sqlp data.csv 'select col1, col2 as friendlyname from data' --format parquet --output data.parquet
 
   # enclose column names with spaces in double quotes
-  qsv sqlp data.csv 'select "col 1", "col 2" from data'
+   qsv sqlp data.csv 'select "col 1", "col 2" from data'
 
-  qsv sqlp data.csv data2.csv 'select * from data join data2 on data.colname = data2.colname'
+   qsv sqlp data.csv data2.csv 'select * from data join data2 on data.colname = data2.colname'
 
-  qsv sqlp data.csv data2.csv 'SELECT col1 FROM data WHERE col1 IN (SELECT col2 FROM data2)'
+   qsv sqlp data.csv data2.csv 'SELECT col1 FROM data WHERE col1 IN (SELECT col2 FROM data2)'
 
-  qsv sqlp data1.csv data2.csv 'SELECT * FROM data1 UNION ALL BY NAME SELECT * FROM data2'
+   qsv sqlp data1.csv data2.csv 'SELECT * FROM data1 UNION ALL BY NAME SELECT * FROM data2'
 
-  qsv sqlp tbl_a.csv tbl_b.csv tbl_c.csv "SELECT * FROM tbl_a \
-    RIGHT ANTI JOIN tbl_b USING (b) \
-    LEFT SEMI JOIN tbl_c USING (c)"
+   qsv sqlp tbl_a.csv tbl_b.csv tbl_c.csv "SELECT * FROM tbl_a \
+     RIGHT ANTI JOIN tbl_b USING (b) \
+     LEFT SEMI JOIN tbl_c USING (c)"
 
   # use "_t_N" aliases to refer to input files, where N is the 1-based index
   # of the input file/s. For example, _t_1 refers to the first input file, _t_2
   # refers to the second input file, and so on.
-  qsv sqlp data.csv data2.csv 'select * from _t_1 join _t_2 on _t_1.colname = _t_2.colname'
+   qsv sqlp data.csv data2.csv 'select * from _t_1 join _t_2 on _t_1.colname = _t_2.colname'
 
-  qsv sqlp data.csv 'SELECT col1, count(*) AS cnt FROM data GROUP BY col1 ORDER BY cnt DESC, col1 ASC'
+   qsv sqlp data.csv 'SELECT col1, count(*) AS cnt FROM data GROUP BY col1 ORDER BY cnt DESC, col1 ASC'
 
-  qsv sqlp data.csv "select lower(col1), substr(col2, 2, 4) from data WHERE starts_with(col1, 'foo')"
+   qsv sqlp data.csv "select lower(col1), substr(col2, 2, 4) from data WHERE starts_with(col1, 'foo')"
 
-  qsv sqlp data.csv "select COALESCE(NULLIF(col2, ''), 'foo') from data"
+   qsv sqlp data.csv "select COALESCE(NULLIF(col2, ''), 'foo') from data"
 
-  qsv sqlp tbl1.csv "SELECT x FROM tbl1 WHERE x IN (SELECT y FROM tbl1)"
+   qsv sqlp tbl1.csv "SELECT x FROM tbl1 WHERE x IN (SELECT y FROM tbl1)"
 
   # Use a SQL script to run a long, complex SQL query or to run SEVERAL SQL queries.
   # When running several queries, each query needs to be separated by a semicolon,
   # the last query will be returned as the result.
   # Typically, earlier queries are used to create tables that can be used in later queries.
   # See test_sqlp/sqlp_boston311_sql_script() for an example.
-  qsv sqlp data.csv data2.csv data3.csv data4.csv script.sql --format json --output data.json
+   qsv sqlp data.csv data2.csv data3.csv data4.csv script.sql --format json --output data.json
 
   # use Common Table Expressions (CTEs) using WITH to simplify complex queries
-  qsv sqlp people.csv "WITH millennials AS (SELECT * FROM people WHERE age >= 25 and age <= 40) \
-    SELECT * FROM millennials WHERE STARTS_WITH(name,'C')"
+   qsv sqlp people.csv "WITH millennials AS (SELECT * FROM people WHERE age >= 25 and age <= 40) \
+     SELECT * FROM millennials WHERE STARTS_WITH(name,'C')"
 
   # CASE statement
-  qsv sqlp data.csv "select CASE WHEN col1 > 10 THEN 'foo' WHEN col1 > 5 THEN 'bar' ELSE 'baz' END from data"
-  qsv sqlp data.csv "select CASE col*5 WHEN 10 THEN 'foo' WHEN 5 THEN 'bar' ELSE 'baz' END from _t_1"
+   qsv sqlp data.csv "select CASE WHEN col1 > 10 THEN 'foo' WHEN col1 > 5 THEN 'bar' ELSE 'baz' END from data"
+   qsv sqlp data.csv "select CASE col*5 WHEN 10 THEN 'foo' WHEN 5 THEN 'bar' ELSE 'baz' END from _t_1"
 
   # spaceship operator: "<=>" (three-way comparison operator)
   #  returns -1 if left < right, 0 if left == right, 1 if left > right
   # https://en.wikipedia.org/wiki/Three-way_comparison#Spaceship_operator
-  qsv sqlp data.csv data2.csv "select data.c2 <=> data2.c2 from data join data2 on data.c1 = data2.c1"
+   qsv sqlp data.csv data2.csv "select data.c2 <=> data2.c2 from data join data2 on data.c1 = data2.c1"
 
   # regex operators: "~" (contains pattern, case-sensitive); "~*" (contains pattern, case-insensitive)
   #   "!~" (does not contain pattern, case-sensitive); "!~*" (does not contain pattern, case-insensitive)
-    qsv sqlp data.csv "select * from data WHERE col1 ~ '^foo' AND col2 > 10"
-    qsv sqlp data.csv "select * from data WHERE col1 !~* 'bar$' AND col2 > 10"
+   qsv sqlp data.csv "select * from data WHERE col1 ~ '^foo' AND col2 > 10"
+   qsv sqlp data.csv "select * from data WHERE col1 !~* 'bar$' AND col2 > 10"
 
   # regexp_like function: regexp_like(<string>, <pattern>, <optional flags>)
   # returns true if <string> matches <pattern>, false otherwise
   #   <optional flags> can be one or more of the following:
   #   'c' (case-sensitive - default), 'i' (case-insensitive), 'm' (multiline)
-  qsv sqlp data.csv "select * from data WHERE regexp_like(col1, '^foo') AND col2 > 10"
+   qsv sqlp data.csv "select * from data WHERE regexp_like(col1, '^foo') AND col2 > 10"
   # case-insensitive regexp_like
-  qsv sqlp data.csv "select * from data WHERE regexp_like(col1, '^foo', 'i') AND col2 > 10"
+   qsv sqlp data.csv "select * from data WHERE regexp_like(col1, '^foo', 'i') AND col2 > 10"
 
   # regexp match using a literal pattern
-    qsv sqlp data.csv "select idx,val from data WHERE val regexp '^foo'"
+   qsv sqlp data.csv "select idx,val from data WHERE val regexp '^foo'"
 
   # regexp match using patterns from another column
-    qsv sqlp data.csv "select idx,val from data WHERE val regexp pattern_col"
+   qsv sqlp data.csv "select idx,val from data WHERE val regexp pattern_col"
 
   # use Parquet, JSONL and Arrow files in SQL queries
-  qsv sqlp data.csv "select * from data join read_parquet('data2.parquet') as t2 ON data.c1 = t2.c1"
-  qsv sqlp data.csv "select * from data join read_ndjson('data2.jsonl') as t2 on data.c1 = t2.c1"
-  qsv sqlp data.csv "select * from data join read_ipc('data2.arrow') as t2 ON data.c1 = t2.c1"
+   qsv sqlp data.csv "select * from data join read_parquet('data2.parquet') as t2 ON data.c1 = t2.c1"
+   qsv sqlp data.csv "select * from data join read_ndjson('data2.jsonl') as t2 on data.c1 = t2.c1"
+   qsv sqlp data.csv "select * from data join read_ipc('data2.arrow') as t2 ON data.c1 = t2.c1"
+
+  # you can also directly load CSVs using the Polars read_csv() SQL function. This is useful when
+  # you want to bypass the regular CSV parser and use Polars' multithreaded, mem-mapped CSV parser
+  # instead - making for dramatically faster queries at the cost of CSV parser configurability.
+  # (i.e. limited to comma delimiter, no CSV comments, etc.)
+   qsv sqlp small_dummy.csv "select * from read_csv('data.csv') order by col1 desc limit 100"
+
+  Note that sqlp will automatically use this "fast path" optimization when there is only 
+  one input CSV file, no CSV parsing options are used, its not a SQL script and the
+  `--no-optimizations` flag is not set.
 
   # use stdin as input
-  cat data.csv | qsv sqlp - 'select * from stdin'
-  cat data.csv | qsv sqlp - data2.csv 'select * from stdin join data2 on stdin.col1 = data2.col1'
+   cat data.csv | qsv sqlp - 'select * from stdin'
+   cat data.csv | qsv sqlp - data2.csv 'select * from stdin join data2 on stdin.col1 = data2.col1'
 
   # automatic snappy decompression/compression
-  qsv sqlp data.csv.sz 'select * from data where col1 > 10' --output result.csv.sz
+   qsv sqlp data.csv.sz 'select * from data where col1 > 10' --output result.csv.sz
 
   # explain query plan
-  qsv sqlp data.csv 'explain select * from data where col1 > 10 order by col2 desc limit 20'
+   qsv sqlp data.csv 'explain select * from data where col1 > 10 order by col2 desc limit 20'
 
 For more examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_sqlp.rs.
 
@@ -116,7 +126,7 @@ sqlp arguments:
     sql                    The SQL query/ies to run. Each input file will be available as a table
                            named after the file name (without the extension), or as "_t_N"
                            where N is the 1-based index.
-                           If the input ends with ".sql", the input will be read as a SQL script file,
+                           If the query ends with ".sql", it will be read as a SQL script file,
                            with each SQL query separated by a semicolon. It will execute the queries
                            in order, and the result of the LAST query will be returned as the result.
 
@@ -124,6 +134,7 @@ sqlp options:
     --format <arg>            The output format to use. Valid values are:
                                 csv      Comma-separated values
                                 json     JSON
+                                jsonl    JSONL (JSON Lines)
                                 parquet  Apache Parquet
                                 arrow    Apache Arrow IPC
                                 avro     Apache Avro
@@ -142,14 +153,23 @@ sqlp options:
                               but will be slower. It will also process LazyFrames in streaming mode.
                               Only use this when you get out of memory errors.
     --no-optimizations        Disable non-default query optimizations. This will make queries slower.
-                              Only use this when you get query errors.
+                              Use this when you get query errors or to force CSV parsing when there
+                              is only one input file, no CSV parsing options are used and its not
+                              a SQL script. Otherwise, the CSV will be read directly into a LazyFrame
+                              using the fast path with the multithreaded, mem-mapped read_csv()
+                              Polars SQL function which is much faster though not as configurable than
+                              the regular CSV parser.
+    --truncate-ragged-lines   Truncate ragged lines when parsing CSVs. If set, rows with more
+                              columns than the header will be truncated. If not set, the query
+                              will fail. Use this only when you get an error about ragged lines.
     --ignore-errors           Ignore errors when parsing CSVs. If set, rows with errors
                               will be skipped. If not set, the query will fail.
-                              Only use this when debugging queries, as polars does batched
+                              Only use this when debugging queries, as Polars does batched
                               parsing and will skip the entire batch where the error occurred.
-    --rnull-values <arg>      The comma-delimited list of strings to consider as null values
-                              when READING CSV files.
-                              (default: <empty string>)                              
+    --rnull-values <arg>      The comma-delimited list of case-sensitive strings to consider as
+                              null values when READING CSV files (e.g. NULL, NONE, <empty string>).
+                              Use "<empty string>" to consider an empty string a null value.
+                              (default: <empty string>)
 
                               CSV OUTPUT FORMAT ONLY:
     --datetime-format <fmt>   The datetime format to use writing datetimes.
@@ -162,10 +182,14 @@ sqlp options:
     --wnull-value <arg>       The string to use when WRITING null values.
                               (default: <empty string>)
 
-                              PARQUET OUTPUT FORMAT ONLY:
-    --compression <arg>       The compression codec to use when writing parquet files.
-                                Valid values are: zstd, lz4raw, gzip, snappy, uncompressed
+                              ARROW/AVRO/PARQUET OUTPUT FORMATS ONLY:
+    --compression <arg>       The compression codec to use when writing arrow or parquet files.
+                                For Arrow, valid values are: zstd, lz4, uncompressed
+                                For Avro, valid values are: deflate, snappy, uncompressed (default)
+                                For Parquet, valid values are: zstd, lz4raw, gzip, snappy, uncompressed
                               (default: zstd)
+
+                              PARQUET OUTPUT FORMAT ONLY:
     --compress-level <arg>    The compression level to use when using zstd or gzip compression.
                               When using zstd, valid values are -7 to 22, with -7 being the
                               lowest compression level and 22 being the highest compression level.
@@ -196,17 +220,19 @@ use std::{
 };
 
 use polars::{
-    io::avro::AvroWriter,
+    io::avro::{AvroWriter, Compression as AvroCompression},
     prelude::{
-        CsvWriter, DataFrame, GzipLevel, IpcWriter, JsonWriter, LazyCsvReader, LazyFileListReader,
-        NullValues, ParquetCompression, ParquetWriter, SerWriter, ZstdLevel,
+        CsvWriter, DataFrame, GzipLevel, IpcCompression, IpcWriter, JsonFormat, JsonWriter,
+        LazyCsvReader, LazyFileListReader, NullValues, ParquetCompression, ParquetWriter,
+        SerWriter, ZstdLevel,
     },
     sql::SQLContext,
 };
+use regex::Regex;
 use serde::Deserialize;
-use tempfile;
 
 use crate::{
+    cmd::joinp::tsvtab_delim,
     config::{Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
     util,
     util::process_input,
@@ -218,26 +244,27 @@ static DEFAULT_ZSTD_COMPRESSION_LEVEL: i32 = 3;
 
 #[derive(Deserialize, Clone)]
 struct Args {
-    arg_input:             Vec<PathBuf>,
-    arg_sql:               String,
-    flag_format:           String,
-    flag_try_parsedates:   bool,
-    flag_infer_len:        Option<usize>,
-    flag_low_memory:       bool,
-    flag_no_optimizations: bool,
-    flag_ignore_errors:    bool,
-    flag_datetime_format:  Option<String>,
-    flag_date_format:      Option<String>,
-    flag_time_format:      Option<String>,
-    flag_float_precision:  Option<usize>,
-    flag_rnull_values:     String,
-    flag_wnull_value:      String,
-    flag_compression:      String,
-    flag_compress_level:   Option<i32>,
-    flag_statistics:       bool,
-    flag_output:           Option<String>,
-    flag_delimiter:        Option<Delimiter>,
-    flag_quiet:            bool,
+    arg_input:                  Vec<PathBuf>,
+    arg_sql:                    String,
+    flag_format:                String,
+    flag_try_parsedates:        bool,
+    flag_infer_len:             Option<usize>,
+    flag_low_memory:            bool,
+    flag_no_optimizations:      bool,
+    flag_ignore_errors:         bool,
+    flag_truncate_ragged_lines: bool,
+    flag_datetime_format:       Option<String>,
+    flag_date_format:           Option<String>,
+    flag_time_format:           Option<String>,
+    flag_float_precision:       Option<usize>,
+    flag_rnull_values:          String,
+    flag_wnull_value:           String,
+    flag_compression:           String,
+    flag_compress_level:        Option<i32>,
+    flag_statistics:            bool,
+    flag_output:                Option<String>,
+    flag_delimiter:             Option<Delimiter>,
+    flag_quiet:                 bool,
 }
 
 #[derive(Default, Clone, PartialEq)]
@@ -245,6 +272,7 @@ enum OutputMode {
     #[default]
     Csv,
     Json,
+    Jsonl,
     Parquet,
     Arrow,
     Avro,
@@ -273,21 +301,8 @@ impl OutputMode {
             }
 
             let w = match args.flag_output {
-                Some(x) => {
-                    let path = Path::new(&x);
-                    // if the output file ends with ".tsv" or ".tab", we use tab as the delimiter
-                    delim = if *self == OutputMode::Csv
-                        && path
-                            .extension()
-                            .map_or(false, |ext| ext.eq_ignore_ascii_case("tsv"))
-                        || path
-                            .extension()
-                            .map_or(false, |ext| ext.eq_ignore_ascii_case("tab"))
-                    {
-                        b'\t'
-                    } else {
-                        delim
-                    };
+                Some(path) => {
+                    delim = tsvtab_delim(path.clone(), delim);
                     Box::new(File::create(path)?) as Box<dyn Write>
                 },
                 None => Box::new(io::stdout()) as Box<dyn Write>,
@@ -304,12 +319,17 @@ impl OutputMode {
                     .with_null_value(args.flag_wnull_value)
                     .include_bom(util::get_envvar_flag("QSV_OUTPUT_BOM"))
                     .finish(&mut df),
-                OutputMode::Json => JsonWriter::new(&mut w).finish(&mut df),
+                OutputMode::Json => JsonWriter::new(&mut w)
+                    .with_json_format(JsonFormat::Json)
+                    .finish(&mut df),
+                OutputMode::Jsonl => JsonWriter::new(&mut w)
+                    .with_json_format(JsonFormat::JsonLines)
+                    .finish(&mut df),
                 OutputMode::Parquet => {
                     let compression: PqtCompression = args
                         .flag_compression
                         .parse()
-                        .unwrap_or(PqtCompression::Lz4Raw);
+                        .unwrap_or(PqtCompression::Uncompressed);
 
                     let parquet_compression = match compression {
                         PqtCompression::Uncompressed => ParquetCompression::Uncompressed,
@@ -337,8 +357,38 @@ impl OutputMode {
                         .finish(&mut df)
                         .map(|_| ())
                 },
-                OutputMode::Arrow => IpcWriter::new(&mut w).finish(&mut df),
-                OutputMode::Avro => AvroWriter::new(&mut w).finish(&mut df),
+                OutputMode::Arrow => {
+                    let compression: ArrowCompression = args
+                        .flag_compression
+                        .parse()
+                        .unwrap_or(ArrowCompression::Uncompressed);
+
+                    let ipc_compression: Option<IpcCompression> = match compression {
+                        ArrowCompression::Uncompressed => None,
+                        ArrowCompression::Lz4 => Some(IpcCompression::LZ4),
+                        ArrowCompression::Zstd => Some(IpcCompression::ZSTD),
+                    };
+
+                    IpcWriter::new(&mut w)
+                        .with_compression(ipc_compression)
+                        .finish(&mut df)
+                },
+                OutputMode::Avro => {
+                    let compression: QsvAvroCompression = args
+                        .flag_compression
+                        .parse()
+                        .unwrap_or(QsvAvroCompression::Uncompressed);
+
+                    let avro_compression = match compression {
+                        QsvAvroCompression::Uncompressed => None,
+                        QsvAvroCompression::Deflate => Some(AvroCompression::Deflate),
+                        QsvAvroCompression::Snappy => Some(AvroCompression::Snappy),
+                    };
+
+                    AvroWriter::new(&mut w)
+                        .with_compression(avro_compression)
+                        .finish(&mut df)
+                },
                 OutputMode::None => Ok(()),
             };
 
@@ -359,9 +409,10 @@ impl FromStr for OutputMode {
     type Err = String;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s.to_lowercase().as_str() {
+        match s.to_ascii_lowercase().as_str() {
             "csv" => Ok(OutputMode::Csv),
             "json" => Ok(OutputMode::Json),
+            "jsonl" => Ok(OutputMode::Jsonl),
             "parquet" => Ok(OutputMode::Parquet),
             "arrow" => Ok(OutputMode::Arrow),
             "avro" => Ok(OutputMode::Avro),
@@ -379,18 +430,59 @@ enum PqtCompression {
     Zstd,
     Lz4Raw,
 }
+#[derive(Default, Copy, Clone)]
+enum ArrowCompression {
+    #[default]
+    Uncompressed,
+    Lz4,
+    Zstd,
+}
+
+#[derive(Default, Copy, Clone)]
+enum QsvAvroCompression {
+    #[default]
+    Uncompressed,
+    Deflate,
+    Snappy,
+}
 
 impl FromStr for PqtCompression {
     type Err = String;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s.to_lowercase().as_str() {
+        match s.to_ascii_lowercase().as_str() {
             "uncompressed" => Ok(PqtCompression::Uncompressed),
             "gzip" => Ok(PqtCompression::Gzip),
             "snappy" => Ok(PqtCompression::Snappy),
             "lz4raw" => Ok(PqtCompression::Lz4Raw),
             "zstd" => Ok(PqtCompression::Zstd),
-            _ => Err(format!("Invalid compression format: {s}")),
+            _ => Err(format!("Invalid Parquet compression format: {s}")),
+        }
+    }
+}
+
+impl FromStr for ArrowCompression {
+    type Err = String;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s.to_ascii_lowercase().as_str() {
+            "uncompressed" => Ok(ArrowCompression::Uncompressed),
+            "lz4" => Ok(ArrowCompression::Lz4),
+            "zstd" => Ok(ArrowCompression::Zstd),
+            _ => Err(format!("Invalid Arrow compression format: {s}")),
+        }
+    }
+}
+
+impl FromStr for QsvAvroCompression {
+    type Err = String;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s.to_ascii_lowercase().as_str() {
+            "uncompressed" => Ok(QsvAvroCompression::Uncompressed),
+            "deflate" => Ok(QsvAvroCompression::Deflate),
+            "snappy" => Ok(QsvAvroCompression::Snappy),
+            _ => Err(format!("Invalid Avro compression format: {s}")),
         }
     }
 }
@@ -406,7 +498,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     } else {
         args.flag_rnull_values
             .split(',')
-            .map(String::from)
+            .map(|value| {
+                if value == "<empty string>" {
+                    String::new()
+                } else {
+                    value.to_string()
+                }
+            })
             .collect()
     };
 
@@ -451,6 +549,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             streaming:           args.flag_low_memory,
             fast_projection:     true,
             eager:               false,
+            row_estimate:        true,
         }
     };
     // gated by log::log_enabled!(log::Level::Debug) to avoid the
@@ -469,41 +568,98 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         );
     }
 
+    // check if the input is a SQL script (ends with .sql)
+    let is_sql_script = std::path::Path::new(&args.arg_sql)
+        .extension()
+        .map_or(false, |ext| ext.eq_ignore_ascii_case("sql"));
+
     let mut ctx = SQLContext::new();
     let mut table_aliases = HashMap::with_capacity(args.arg_input.len());
     let mut lossy_table_name = Cow::default();
     let mut table_name;
 
-    for (idx, table) in args.arg_input.iter().enumerate() {
-        // as we are using the table name as alias, we need to make sure that the table name is a
-        // valid identifier. if its not utf8, we use the lossy version
-        table_name = Path::new(table)
+    // if there is only one input file and its a CSV and no CSV parsing options are used,
+    // we can use the fast path to read the CSV directly into a LazyFrame without having to
+    // parse and register it as a table in the SQL context using Polars SQL's read_csv function
+    if args.arg_input.len() == 1
+        && !is_sql_script
+        && delim == b','
+        && !args.flag_no_optimizations
+        && !args.flag_try_parsedates
+        && args.flag_infer_len != Some(250)
+        && !args.flag_low_memory
+        && !args.flag_truncate_ragged_lines
+        && !args.flag_ignore_errors
+        && args.flag_rnull_values.is_empty()
+        && comment_char.is_none()
+        && std::path::Path::new(&args.arg_input[0])
+            .extension()
+            .map_or(false, |ext| ext.eq_ignore_ascii_case("csv"))
+    {
+        // replace all instances of the FROM clause case-insensitive in the SQL query with the
+        // read_csv function using a regex
+        let input = &args.arg_input[0];
+        let table_name = Path::new(input)
             .file_stem()
             .and_then(std::ffi::OsStr::to_str)
             .unwrap_or_else(|| {
-                lossy_table_name = table.to_string_lossy();
+                lossy_table_name = input.to_string_lossy();
                 &lossy_table_name
             });
-        table_aliases.insert(table_name.to_string(), format!("_t_{}", idx + 1));
-
+        let sql = args.arg_sql.clone();
+        // the regex is case-insensitive and allows for the table name to be enclosed in single or
+        // double quotes or not enclosed at all. It also allows for the table name to be
+        // aliased as _t_1.
+        let from_clause_regex =
+            Regex::new(&format!(r#"(?i)FROM\s+['"]?({table_name}|_t_1)['"]?"#))?;
+        let modified_query = from_clause_regex.replace_all(
+            &sql,
+            format!("FROM read_csv('{}')", input.to_string_lossy()),
+        );
+        args.arg_sql = modified_query.to_string();
         if debuglog_flag {
-            log::debug!(
-                "Registering table: {table_name} as {alias}",
-                alias = table_aliases.get(table_name).unwrap(),
-            );
+            log::debug!("Using fast path - Modified Query: {modified_query}");
+        }
+    } else {
+        if debuglog_flag {
+            // Using the slow path to read and parse the CSV/s into tables in the SQL context.
+            log::debug!("Using the slow path...");
+        }
+        // we have more than one input and/or we are using CSV parsing options, so we need to
+        // parse the CSV first, and register the input files as tables in the SQL context
+        for (idx, table) in args.arg_input.iter().enumerate() {
+            // as we are using the table name as alias, we need to make sure that the table name is
+            // a valid identifier. if its not utf8, we use the lossy version
+            table_name = Path::new(table)
+                .file_stem()
+                .and_then(std::ffi::OsStr::to_str)
+                .unwrap_or_else(|| {
+                    lossy_table_name = table.to_string_lossy();
+                    &lossy_table_name
+                });
+
+            table_aliases.insert(table_name.to_string(), format!("_t_{}", idx + 1));
+
+            if debuglog_flag {
+                log::debug!(
+                    "Registering table: {table_name} as {alias}",
+                    alias = table_aliases.get(table_name).unwrap(),
+                );
+            }
+            let lf = LazyCsvReader::new(table)
+                .has_header(true)
+                .with_missing_is_null(true)
+                .with_comment_prefix(comment_char.as_deref())
+                .with_null_values(Some(NullValues::AllColumns(rnull_values.clone())))
+                .with_separator(tsvtab_delim(table, delim))
+                .with_infer_schema_length(args.flag_infer_len)
+                .with_try_parse_dates(args.flag_try_parsedates)
+                .with_ignore_errors(args.flag_ignore_errors)
+                .truncate_ragged_lines(args.flag_truncate_ragged_lines)
+                .low_memory(args.flag_low_memory)
+                .finish()?;
+            ctx.register(table_name, lf.with_optimizations(optimization_state));
         }
-        let lf = LazyCsvReader::new(table)
-            .has_header(true)
-            .with_missing_is_null(true)
-            .with_comment_prefix(comment_char.as_deref())
-            .with_null_values(Some(NullValues::AllColumns(rnull_values.clone())))
-            .with_separator(delim)
-            .with_infer_schema_length(args.flag_infer_len)
-            .with_try_parse_dates(args.flag_try_parsedates)
-            .with_ignore_errors(args.flag_ignore_errors)
-            .low_memory(args.flag_low_memory)
-            .finish()?;
-        ctx.register(table_name, lf.with_optimizations(optimization_state));
     }
 
     if debuglog_flag {
@@ -512,10 +668,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     }
 
     // check if the query is a SQL script
-    let queries = if std::path::Path::new(&args.arg_sql)
-        .extension()
-        .map_or(false, |ext| ext.eq_ignore_ascii_case("sql"))
-    {
+    let queries = if is_sql_script {
         let mut file = File::open(&args.arg_sql)?;
         let mut sql_script = String::new();
         file.read_to_string(&mut sql_script)?;
@@ -530,7 +683,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     };
 
     if debuglog_flag {
-        log::debug!("SQL query/ies: {queries:?}");
+        log::debug!("SQL query/ies({}): {queries:?}", queries.len());
     }
 
     let num_queries = queries.len();
