@@ -3,7 +3,7 @@ Compute summary statistics & infers data types for each column in a CSV.
 
 Summary statistics includes sum, min/max/range, min/max length, mean, stddev, variance,
 nullcount, sparsity, quartiles, interquartile range (IQR), lower/upper fences, skewness, median, 
-cardinality, mode/s & antimode/s, and median absolute deviation (MAD). Note that some
+cardinality, mode/s & "antimode/s", and median absolute deviation (MAD). Note that some
 statistics requires loading the entire file into memory, so they must be enabled explicitly. 
 
 By default, the following "streaming" statistics are reported for *every* column:
@@ -74,7 +74,7 @@ stats options:
                               See 'qsv select --help' for the format details.
                               This is provided here because piping 'qsv select'
                               into 'qsv stats' will disable the use of indexing.
-    --everything              Show all statistics available.
+    -E, --everything          Show all statistics available.
     --typesonly               Infer data types only and do not compute statistics.
                               Note that if you want to infer dates, you'll still need to use
                               the --infer-dates and --dates-whitelist options.
@@ -95,6 +95,7 @@ stats options:
                               This requires loading all CSV data in memory.
     --round <decimal_places>  Round statistics to <decimal_places>. Rounding is done following
                               Midpoint Nearest Even (aka "Bankers Rounding") rule.
+                              https://docs.rs/rust_decimal/latest/rust_decimal/enum.RoundingStrategy.html
                               For dates - range, stddev & IQR are always at least 5 decimal places as
                               they are reported in days, and 5 places gives us millisecond precision.
                               [default: 4]
@@ -114,12 +115,12 @@ stats options:
 
                               Note that false positive date matches WILL most likely occur
                               when using "all" as unix epoch timestamps are just numbers.
-                              Be sure to only use "all" if you know the columns you're
+                              Be sure to only use "all" if you know ALL the columns you're
                               inspecting are dates, boolean or string fields.
-                              
+
                               To avoid false positives, preprocess the file first 
-                              with `apply datefmt` to convert unix epoch timestamp columns
-                              to RFC3339 format.
+                              with the `datefmt` command to convert unix epoch timestamp
+                              columns to RFC3339 format.
                               [default: date,time,due,open,close,created]
     --prefer-dmy              Parse dates in dmy format. Otherwise, use mdy format.
                               Ignored if --infer-dates is false.
@@ -134,7 +135,7 @@ stats options:
                               by other qsv commands (currently `schema` & `tojsonl`) to
                               load cached stats into memory faster. If set, the binary
                               encoded stats will be written to <FILESTEM>.stats.csv.bin.sz.
-                              You can preemtively create the binary encoded stats file
+                              You can preemptively create the binary encoded stats file
                               by using this option before running the `schema` and `tojsonl`
                               commands and they will automatically load the binary encoded
                               stats file if it exists.
@@ -162,7 +163,7 @@ for Datapusher+ (https://github.com/dathere/datapusher-plus).
 
 It underpins the `schema` and `validate` commands - enabling the automatic creation of
 a JSONschema based on a CSV's summary statistics; and use the generated JSONschema to
-quickly validate complex CSVs (NYC's 311 data) at almost 350,000 records/sec.
+quickly validate complex CSVs (NYC's 311 data) at almost 930,000 records/sec.
 
 It's type inferences are also used by the `tojsonl` command to generate properly typed
 JSONL files.
@@ -172,17 +173,13 @@ with >480 tests.
 */
 
 use std::{
-    borrow::ToOwned,
     default::Default,
     fmt, fs, io,
     io::Write,
     iter::repeat,
     path::{Path, PathBuf},
     str,
-    sync::{
-        atomic::{AtomicBool, Ordering},
-        OnceLock,
-    },
+    sync::OnceLock,
 };
 
 use gzp::{par::compress::ParCompressBuilder, snap::Snap};
@@ -260,11 +257,11 @@ struct StatsArgs {
 }
 
 static INFER_DATE_FLAGS: OnceLock<Vec<bool>> = OnceLock::new();
-static DMY_PREFERENCE: AtomicBool = AtomicBool::new(false);
 static RECORD_COUNT: OnceLock<u64> = OnceLock::new();
 
 // number of milliseconds per day
 const MS_IN_DAY: f64 = 86_400_000.0;
+const MS_IN_DAY_INT: i64 = 86_400_000;
 // number of decimal places when rounding days
 // 5 decimal places give us millisecond precision
 const DAY_DECIMAL_PLACES: u32 = 5;
@@ -285,6 +282,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         args.flag_cardinality = true;
     }
 
+    // check prefer_dmy env var
+    args.flag_prefer_dmy = args.flag_prefer_dmy || util::get_envvar_flag("QSV_PREFER_DMY");
+
     // set stdout output flag
     let stdout_output_flag = args.flag_output.is_none();
 
@@ -315,7 +315,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             false
         } else {
             let p = args.flag_output.clone().unwrap();
-            p.to_lowercase().ends_with(".sz")
+            p.to_ascii_lowercase().ends_with(".sz")
         },
         canonical_input_path: String::new(),
         canonical_stats_path: String::new(),
@@ -373,7 +373,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     if let Some(path) = fconfig.path.clone() {
         let path_file_stem = path.file_stem().unwrap().to_str().unwrap();
-        let stats_file = stats_path(&path, false);
+        let stats_file = stats_path(&path, false)?;
         // check if <FILESTEM>.stats.csv file already exists.
         // If it does, check if it was compiled using the same args.
         // However, if the --force flag is set,
@@ -493,7 +493,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
             // clone a copy of stats so we can binary encode it to disk later
             if write_stats_binout {
-                stats_for_encoding = stats.clone();
+                stats_for_encoding.clone_from(&stats);
             }
 
             let stats_sr_vec = args.stats_to_records(stats);
@@ -510,14 +510,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 wtr.write_record(vec![&*header].into_iter().chain(stat))?;
             }
 
-            // update the stats args json file
-            if write_stats_binout {
-                current_stats_args.canonical_input_path =
-                    path.canonicalize()?.to_str().unwrap().to_string();
-                current_stats_args.record_count = *record_count;
-                current_stats_args.compute_duration_ms = start_time.elapsed().as_millis() as u64;
-                current_stats_args.date_generated = chrono::Utc::now().to_rfc3339();
-            }
+            // update the stats args json metadata
+            current_stats_args.canonical_input_path =
+                path.canonicalize()?.to_str().unwrap().to_string();
+            current_stats_args.record_count = *record_count;
+            current_stats_args.compute_duration_ms = start_time.elapsed().as_millis() as u64;
+            current_stats_args.date_generated = chrono::Utc::now().to_rfc3339();
         }
     }
 
@@ -533,7 +531,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         stats_csv_tempfile_fname
     } else {
         // we didn't compute the stats, re-use the existing stats file
-        stats_path(fconfig.path.as_ref().unwrap(), false)
+        stats_path(fconfig.path.as_ref().unwrap(), false)?
             .to_str()
             .unwrap()
             .to_owned()
@@ -541,7 +539,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     if fconfig.is_stdin() {
         // if we read from stdin, copy the temp stats file to "stdin.stats.csv"
-        let mut stats_pathbuf = stats_path(fconfig.path.as_ref().unwrap(), true);
+        let mut stats_pathbuf = stats_path(fconfig.path.as_ref().unwrap(), true)?;
         fs::copy(currstats_filename.clone(), stats_pathbuf.clone())?;
 
         // save the stats args to "stdin.stats.csv.json"
@@ -559,42 +557,45 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             fs::copy(currstats_filename.clone(), stats_pathbuf.clone())?;
         }
 
-        // save the stats args to "<FILESTEM>.stats.csv.json"
-        stats_pathbuf.set_extension("csv.json");
-        // write empty file first so we can canonicalize it
-        std::fs::File::create(stats_pathbuf.clone())?;
-        current_stats_args.canonical_stats_path = stats_pathbuf
-            .clone()
-            .canonicalize()?
-            .to_str()
-            .unwrap()
-            .to_string();
-        std::fs::write(
-            stats_pathbuf.clone(),
-            // safety: we know that current_stats_args is JSON serializable
-            serde_json::to_string_pretty(&current_stats_args).unwrap(),
-        )?;
+        if compute_stats {
+            // save the stats args to "<FILESTEM>.stats.csv.json"
+            // if we computed the stats
+            stats_pathbuf.set_extension("csv.json");
+            // write empty file first so we can canonicalize it
+            std::fs::File::create(stats_pathbuf.clone())?;
+            current_stats_args.canonical_stats_path = stats_pathbuf
+                .clone()
+                .canonicalize()?
+                .to_str()
+                .unwrap()
+                .to_string();
+            std::fs::write(
+                stats_pathbuf.clone(),
+                // safety: we know that current_stats_args is JSON serializable
+                serde_json::to_string_pretty(&current_stats_args).unwrap(),
+            )?;
 
-        // only create the binary encoded files if we computed the stats
-        // and the user specified --stats-binout
-        if compute_stats && write_stats_binout {
-            // binary encode the stats to "<FILESTEM>.stats.csv.bin.sz"
-            let mut stats_pathbuf = path;
-            stats_pathbuf.set_extension("stats.csv.bin.sz");
-            // we do the binary encoding inside a block so that the encoded_file
-            // automatically gets dropped/flushed before we copy it to the output file
-            {
-                let encoded_file = ParCompressBuilder::<Snap>::new()
-                    .num_threads(util::max_jobs())?
-                    .buffer_size(DEFAULT_WTR_BUFFER_CAPACITY * 2)?
-                    .pin_threads(Some(0))
-                    .from_writer(fs::File::create(stats_pathbuf.clone())?);
+            // only create the binary encoded files if we computed the stats
+            // and the user specified --stats-binout
+            if write_stats_binout {
+                // binary encode the stats to "<FILESTEM>.stats.csv.bin.sz"
+                let mut stats_bin_pathbuf = path;
+                stats_bin_pathbuf.set_extension("stats.csv.bin.sz");
+                // we do the binary encoding inside a block so that the encoded_file
+                // automatically gets dropped/flushed before we copy it to the output file
+                {
+                    let encoded_file = ParCompressBuilder::<Snap>::new()
+                        .num_threads(util::max_jobs())?
+                        .buffer_size(DEFAULT_WTR_BUFFER_CAPACITY * 2)?
+                        .pin_threads(Some(0))
+                        .from_writer(fs::File::create(stats_bin_pathbuf.clone())?);
 
-                if let Err(e) = bincode::serialize_into(encoded_file, &stats_for_encoding) {
-                    return fail_clierror!(
-                        "Failed to write binary encoded stats {}: {e:?}",
-                        stats_pathbuf.display()
-                    );
+                    if let Err(e) = bincode::serialize_into(encoded_file, &stats_for_encoding) {
+                        return fail_clierror!(
+                            "Failed to write binary encoded stats {}: {e:?}",
+                            stats_bin_pathbuf.display()
+                        );
+                    }
                 }
             }
         }
@@ -621,12 +622,7 @@ impl Args {
         let mut rdr = self.rconfig().reader()?;
         let (headers, sel) = self.sel_headers(&mut rdr)?;
 
-        init_date_inference(
-            self.flag_infer_dates,
-            self.flag_prefer_dmy,
-            &headers,
-            whitelist,
-        )?;
+        init_date_inference(self.flag_infer_dates, &headers, whitelist)?;
 
         let stats = self.compute(&sel, rdr.byte_records());
         Ok((headers, stats))
@@ -646,12 +642,7 @@ impl Args {
         let mut rdr = self.rconfig().reader()?;
         let (headers, sel) = self.sel_headers(&mut rdr)?;
 
-        init_date_inference(
-            self.flag_infer_dates,
-            self.flag_prefer_dmy,
-            &headers,
-            whitelist,
-        )?;
+        init_date_inference(self.flag_infer_dates, &headers, whitelist)?;
 
         let chunk_size = util::chunk_size(idx_count as usize, util::njobs(self.flag_jobs));
         let nchunks = util::num_of_chunks(idx_count as usize, chunk_size);
@@ -712,8 +703,9 @@ impl Args {
 
         // safety: we know INFER_DATE_FLAGS is Some because we called init_date_inference
         let infer_date_flags = INFER_DATE_FLAGS.get().unwrap();
-        // so we don't need to get flag_infer_boolean from big args struct for each iteration
+        // so we don't need to get infer_boolean/prefer_dmy from big args struct for each iteration
         let infer_boolean = self.flag_infer_boolean;
+        let prefer_dmy = self.flag_prefer_dmy;
 
         // safety: because we're using iterators and INFER_DATE_FLAGS has the same size,
         // we know we don't need to bounds check
@@ -726,6 +718,7 @@ impl Args {
                         field,
                         *infer_date_flags.get_unchecked(i),
                         infer_boolean,
+                        prefer_dmy,
                     );
                     i += 1;
                 }
@@ -832,46 +825,27 @@ impl Args {
     }
 }
 
-// returns the path to the stats file
-// safety: unwraps are safe because we know stats_csv_path is a valid path
-fn stats_path(stats_csv_path: &Path, stdin_flag: bool) -> PathBuf {
-    let mut p = stats_csv_path
-        .to_path_buf()
-        .into_os_string()
-        .into_string()
-        .unwrap();
-
-    let fname = stats_csv_path
-        .file_name()
-        .unwrap()
-        .to_os_string()
-        .into_string()
-        .unwrap();
+/// returns the path to the stats file
+fn stats_path(stats_csv_path: &Path, stdin_flag: bool) -> io::Result<PathBuf> {
+    let parent = stats_csv_path
+        .parent()
+        .ok_or_else(|| io::Error::new(io::ErrorKind::InvalidInput, "Invalid path"))?;
     let fstem = stats_csv_path
         .file_stem()
-        .unwrap()
-        .to_os_string()
-        .into_string()
-        .unwrap();
+        .ok_or_else(|| io::Error::new(io::ErrorKind::InvalidInput, "Invalid file name"))?;
 
-    if let Some(nofn) = p.strip_suffix(&fname) {
-        p = nofn.to_string();
+    let new_fname = if stdin_flag {
+        "stdin.stats.csv".to_string()
     } else {
-        p = String::new();
-    }
-    if stdin_flag {
-        p.push_str("stdin.stats.csv");
-    } else {
-        p.push_str(&format!("{fstem}.stats.csv"));
-    }
+        format!("{}.stats.csv", fstem.to_string_lossy())
+    };
 
-    PathBuf::from(&p)
+    Ok(parent.join(new_fname))
 }
 
 #[inline]
 fn init_date_inference(
     infer_dates: bool,
-    prefer_dmy: bool,
     headers: &csv::ByteRecord,
     flag_whitelist: &str,
 ) -> Result<(), String> {
@@ -883,18 +857,15 @@ fn init_date_inference(
         return Ok(());
     }
 
-    let dmy_preferred = prefer_dmy || util::get_envvar_flag("QSV_PREFER_DMY");
-    DMY_PREFERENCE.store(dmy_preferred, Ordering::Relaxed);
-
-    let whitelist_lower = flag_whitelist.to_lowercase();
-    log::info!("inferring dates with date-whitelist: {whitelist_lower}");
-
-    let infer_date_flags = if whitelist_lower == "all" {
-        log::info!("inferring dates for ALL fields with DMY preference: {dmy_preferred}");
+    let infer_date_flags = if flag_whitelist.eq_ignore_ascii_case("all") {
+        log::info!("inferring dates for ALL fields");
         vec![true; headers.len()]
     } else {
         let mut header_str = String::new();
         let mut date_found = false;
+        let whitelist_lower = flag_whitelist.to_lowercase();
+        log::info!("inferring dates with date-whitelist: {whitelist_lower}");
+
         let whitelist = whitelist_lower
             .split(',')
             .map(str::trim)
@@ -907,9 +878,7 @@ fn init_date_inference(
                     .iter()
                     .any(|whitelist_item| header_str.contains(whitelist_item));
                 if date_found {
-                    log::info!(
-                        "inferring dates for {header_str} with DMY preference: {dmy_preferred}"
-                    );
+                    log::info!("inferring dates for {header_str}");
                 }
                 date_found
             })
@@ -959,13 +928,9 @@ pub struct Stats {
 
 #[inline]
 fn timestamp_ms_to_rfc3339(timestamp: i64, typ: FieldType) -> String {
-    use chrono::{DateTime, NaiveDateTime, Utc};
-
-    let date_val = DateTime::<Utc>::from_naive_utc_and_offset(
-        NaiveDateTime::from_timestamp_millis(timestamp).unwrap_or_default(),
-        Utc,
-    )
-    .to_rfc3339();
+    let date_val = chrono::DateTime::from_timestamp_millis(timestamp)
+        .unwrap_or_default()
+        .to_rfc3339();
 
     // if type = Date, only return the date component
     // do not return the time component
@@ -1014,8 +979,9 @@ impl Stats {
     }
 
     #[inline]
-    fn add(&mut self, sample: &[u8], infer_dates: bool, infer_boolean: bool) {
-        let (sample_type, timestamp_val) = FieldType::from_sample(infer_dates, sample, self.typ);
+    fn add(&mut self, sample: &[u8], infer_dates: bool, infer_boolean: bool, prefer_dmy: bool) {
+        let (sample_type, timestamp_val) =
+            FieldType::from_sample(infer_dates, prefer_dmy, sample, self.typ);
         self.typ.merge(sample_type);
 
         // we're inferring --typesonly, so don't add samples to compute statistics
@@ -1454,6 +1420,7 @@ impl Commute for Stats {
 }
 
 #[allow(clippy::enum_variant_names)]
+#[allow(clippy::unsafe_derive_deserialize)]
 #[derive(Clone, Copy, PartialEq, Default, Serialize, Deserialize)]
 enum FieldType {
     // The default - TNull, is the most specific type.
@@ -1477,6 +1444,7 @@ impl FieldType {
     #[inline]
     pub fn from_sample(
         infer_dates: bool,
+        prefer_dmy: bool,
         sample: &[u8],
         current_type: FieldType,
     ) -> (FieldType, Option<i64>) {
@@ -1489,41 +1457,59 @@ impl FieldType {
             return (FieldType::TString, None);
         }
 
-        let Ok(string) = from_utf8(sample) else {
-            // if the string is not valid utf8, we assume it is a binary string
-            // and return a string type
-            return (FieldType::TString, None);
-        };
-
-        if current_type == FieldType::TFloat
+        let utf8_string = if current_type == FieldType::TFloat
             || current_type == FieldType::TInteger
             || current_type == FieldType::TNull
         {
             if let Ok(int_val) = atoi_simd::parse::<i64>(sample) {
-                // leading zero, its a string (e.g. zip codes)
-                if string.as_bytes()[0] == b'0' && int_val != 0 {
+                // safety: we know sample is not empty
+                if int_val != 0 && unsafe { sample.get_unchecked(0) == &b'0' } {
+                    // leading zero, its a string (e.g. zip codes)
                     return (TString, None);
                 }
                 return (TInteger, None);
             }
 
+            let Ok(string) = from_utf8(sample) else {
+                // if the string is not valid utf8, we assume it is a binary string
+                // and return a string type
+                return (FieldType::TString, None);
+            };
+
             if string.parse::<f64>().is_ok() {
                 return (TFloat, None);
             }
-        }
+
+            Some(string)
+        } else {
+            None
+        };
 
         if infer_dates
             && (current_type == FieldType::TDate
                 || current_type == FieldType::TDateTime
                 || current_type == FieldType::TNull)
         {
-            if let Ok(parsed_date) =
-                parse_with_preference(string, DMY_PREFERENCE.load(Ordering::Relaxed))
-            {
-                // get date in rfc3339 format, if it ends with "T00:00:00+00:00"
-                // its a Date type, otherwise, its DateTime.
+            // if we already have a utf8 string, use it, otherwise, convert sample to utf8
+            // if the sample isn't valid utf8, we assume a binary string & return a string type
+            let date_string = if let Some(s) = utf8_string {
+                s
+            } else if let Ok(s) = from_utf8(sample) {
+                s
+            } else {
+                // the string is not valid utf8, we assume it is a binary string
+                // and return a string type
+                return (FieldType::TString, None);
+            };
+
+            if let Ok(parsed_date) = parse_with_preference(date_string, prefer_dmy) {
+                // check if the tstamp (Unix Epoch format) modulo by 86400000 (ms in a day)
+                // if the remainder is 0, we says its Date type candidate, otherwise, its DateTime.
+                // this is a performance optimization to avoid parsing the date to rfc3339
+                // and then checking if the time component is T00:00:00, as was done previously
+                // see https://stackoverflow.com/questions/40948290/is-it-safe-to-use-modulo-operator-with-unix-epoch-timestamp
                 let ts_val = parsed_date.timestamp_millis();
-                if parsed_date.to_rfc3339().ends_with("T00:00:00+00:00") {
+                if ts_val % MS_IN_DAY_INT == 0 {
                     return (TDate, Some(ts_val));
                 }
                 return (TDateTime, Some(ts_val));
@@ -1570,6 +1556,7 @@ impl fmt::Display for FieldType {
     }
 }
 
+#[cfg(debug_assertions)]
 impl fmt::Debug for FieldType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
@@ -1695,7 +1682,10 @@ impl TypedMinMax {
                 #[allow(clippy::cast_precision_loss)]
                 self.floats.add(n as f64);
             },
-            TDate | TDateTime => {
+            // it must be a TDate or TDateTime
+            // we use _ here for the match to avoid
+            // the overhead of matching on the OR value, however minor
+            _ => {
                 let n = atoi_simd::parse::<i64>(sample).unwrap();
                 self.dates.add(n);
             },
