@@ -1,5 +1,4 @@
 use std::{
-    borrow::ToOwned,
     env, fs,
     io::{self, Read},
     path::{Path, PathBuf},
@@ -78,7 +77,7 @@ pub struct Config {
     select_columns:    Option<SelectColumns>,
     delimiter:         u8,
     pub no_headers:    bool,
-    flexible:          bool,
+    pub flexible:      bool,
     terminator:        csv::Terminator,
     pub quote:         u8,
     quote_style:       csv::QuoteStyle,
@@ -89,8 +88,10 @@ pub struct Config {
     trim:              csv::Trim,
     autoindex_size:    u64,
     prefer_dmy:        bool,
-    comment:           Option<u8>,
+    pub comment:       Option<u8>,
     snappy:            bool, // flag to enable snappy compression/decompression
+    pub read_buffer:   u32,
+    pub write_buffer:  u32,
 }
 
 // Empty trait as an alias for Seek and Read that avoids auto trait errors
@@ -105,6 +106,22 @@ impl Config {
         };
         let (path, mut delim, snappy) = match *path {
             None => (None, default_delim, false),
+            // WIP: support remote files; currently only http(s) is supported
+            // Some(ref s) if s.starts_with("http") && Url::parse(s).is_ok() => {
+            //     let mut snappy = false;
+            //     let delim = if s.ends_with(".csv.sz") {
+            //         snappy = true;
+            //         b','
+            //     } else if s.ends_with(".tsv.sz") || s.ends_with(".tab.sz") {
+            //         snappy = true;
+            //         b'\t'
+            //     } else {
+            //         default_delim
+            //     };
+            //     // download the file to a temporary location
+            //     util::download_file()
+            //     (Some(PathBuf::from(s)), delim, snappy)
+            // },
             Some(ref s) if &**s == "-" => (None, default_delim, false),
             Some(ref s) => {
                 let path = PathBuf::from(s);
@@ -113,7 +130,7 @@ impl Config {
                     .unwrap_or_default()
                     .to_str()
                     .unwrap()
-                    .to_lowercase();
+                    .to_ascii_lowercase();
                 let mut snappy = false;
                 let delim = if file_extension == "tsv" || file_extension == "tab" {
                     b'\t'
@@ -136,6 +153,11 @@ impl Config {
         };
         let sniff = util::get_envvar_flag("QSV_SNIFF_DELIMITER")
             || util::get_envvar_flag("QSV_SNIFF_PREAMBLE");
+        let comment: Option<u8> = match env::var("QSV_COMMENT_CHAR") {
+            Ok(comment_char) => Some(comment_char.as_bytes().first().unwrap().to_owned()),
+            Err(_) => None,
+        };
+        let no_headers = util::get_envvar_flag("QSV_NO_HEADERS");
         let mut preamble = 0_u64;
         if sniff && path.is_some() {
             let sniff_path = path.as_ref().unwrap().to_str().unwrap();
@@ -165,7 +187,7 @@ impl Config {
             idx_path: None,
             select_columns: None,
             delimiter: delim,
-            no_headers: false,
+            no_headers,
             flexible: false,
             terminator: csv::Terminator::Any(b'\n'),
             quote: b'"',
@@ -180,8 +202,16 @@ impl Config {
                 .parse()
                 .unwrap_or(0),
             prefer_dmy: util::get_envvar_flag("QSV_PREFER_DMY"),
-            comment: None,
+            comment,
             snappy,
+            read_buffer: std::env::var("QSV_RDR_BUFFER_CAPACITY")
+                .unwrap_or_else(|_| DEFAULT_RDR_BUFFER_CAPACITY.to_string())
+                .parse()
+                .unwrap_or(DEFAULT_RDR_BUFFER_CAPACITY as u32),
+            write_buffer: std::env::var("QSV_WTR_BUFFER_CAPACITY")
+                .unwrap_or_else(|_| DEFAULT_WTR_BUFFER_CAPACITY.to_string())
+                .parse()
+                .unwrap_or(DEFAULT_WTR_BUFFER_CAPACITY as u32),
         }
     }
 
@@ -209,11 +239,7 @@ impl Config {
         if env::var("QSV_TOGGLE_HEADERS").unwrap_or_else(|_| "0".to_owned()) == "1" {
             yes = !yes;
         }
-        if util::get_envvar_flag("QSV_NO_HEADERS") {
-            self.no_headers = true;
-        } else {
-            self.no_headers = yes;
-        }
+        self.no_headers = yes;
         self
     }
 
@@ -268,6 +294,16 @@ impl Config {
         self
     }
 
+    pub const fn read_buffer(mut self, buffer: u32) -> Config {
+        self.read_buffer = buffer;
+        self
+    }
+
+    pub const fn write_buffer(mut self, buffer: u32) -> Config {
+        self.write_buffer = buffer;
+        self
+    }
+
     #[allow(clippy::missing_const_for_fn)]
     pub fn select(mut self, sel_cols: SelectColumns) -> Config {
         self.select_columns = Some(sel_cols);
@@ -278,10 +314,9 @@ impl Config {
         self.path.is_none()
     }
 
-    // comment out for now until we have a use case for this
-    // pub const fn is_snappy(&self) -> bool {
-    //     self.snappy
-    // }
+    pub const fn is_snappy(&self) -> bool {
+        self.snappy
+    }
 
     #[inline]
     pub fn selection(&self, first_record: &csv::ByteRecord) -> Result<Selection, String> {
@@ -327,7 +362,7 @@ impl Config {
     pub fn reader_file_stdin(&self) -> io::Result<csv::Reader<Box<dyn SeekRead + 'static>>> {
         Ok(match self.path {
             None => {
-                // Create a buffer in memory when stdin needs to be indexed
+                // Create a buffer in memory for stdin
                 let mut buffer: Vec<u8> = Vec::new();
                 let stdin = io::stdin();
                 stdin.lock().read_to_end(&mut buffer)?;
@@ -486,16 +521,6 @@ impl Config {
 
     #[allow(clippy::wrong_self_convention)]
     pub fn from_reader<R: Read>(&self, rdr: R) -> csv::Reader<R> {
-        let rdr_capacitys = env::var("QSV_RDR_BUFFER_CAPACITY")
-            .unwrap_or_else(|_| DEFAULT_RDR_BUFFER_CAPACITY.to_string());
-        let rdr_buffer: usize = rdr_capacitys.parse().unwrap_or(DEFAULT_RDR_BUFFER_CAPACITY);
-
-        let rdr_comment: Option<u8> = if let Ok(comment_char) = env::var("QSV_COMMENT_CHAR") {
-            Some(comment_char.as_bytes().first().unwrap().to_owned())
-        } else {
-            self.comment
-        };
-
         csv::ReaderBuilder::new()
             .flexible(self.flexible)
             .delimiter(self.delimiter)
@@ -503,8 +528,8 @@ impl Config {
             .quote(self.quote)
             .quoting(self.quoting)
             .escape(self.escape)
-            .buffer_capacity(rdr_buffer)
-            .comment(rdr_comment)
+            .buffer_capacity(self.read_buffer as usize)
+            .comment(self.comment)
             .trim(self.trim)
             .from_reader(rdr)
     }
@@ -529,10 +554,6 @@ impl Config {
 
     #[allow(clippy::wrong_self_convention)]
     pub fn from_writer<W: io::Write>(&self, mut wtr: W) -> csv::Writer<W> {
-        let wtr_capacitys = env::var("QSV_WTR_BUFFER_CAPACITY")
-            .unwrap_or_else(|_| DEFAULT_WTR_BUFFER_CAPACITY.to_string());
-        let wtr_buffer: usize = wtr_capacitys.parse().unwrap_or(DEFAULT_WTR_BUFFER_CAPACITY);
-
         if util::get_envvar_flag("QSV_OUTPUT_BOM") {
             wtr.write_all("\u{FEFF}".as_bytes()).unwrap();
         }
@@ -545,7 +566,7 @@ impl Config {
             .quote_style(self.quote_style)
             .double_quote(self.double_quote)
             .escape(self.escape.unwrap_or(b'\\'))
-            .buffer_capacity(wtr_buffer)
+            .buffer_capacity(self.write_buffer as usize)
             .from_writer(wtr)
     }
 }
