@@ -1,5 +1,5 @@
 #![cfg_attr(
-    feature = "cargo-clippy",
+    clippy,
     allow(
         clippy::cast_possible_truncation,
         clippy::cast_possible_wrap,
@@ -55,6 +55,7 @@ macro_rules! command_list {
         "
     applydp     Apply series of transformations to a column
     count       Count records
+    datefmt     Format date/datetime strings
     describegpt Infer extended metadata using a LLM
     diff        Find the difference between two CSVs
     dedup       Remove redundant rows
@@ -66,10 +67,12 @@ macro_rules! command_list {
     help        Show this usage message
     index       Create CSV index for faster access
     input       Read CSVs w/ special quoting, skipping, trimming & transcoding rules
+    joinp       Join CSV files using the Pola.rs engine
     luau        Execute Luau script on CSV data
     pseudo      Pseudonymise the values of a column
     rename      Rename the columns of CSV data efficiently
     replace     Replace patterns in CSV data
+    reverse     Reverse rows of CSV data
     safenames   Modify a CSV's header names to db-safe names
     sample      Randomly sample CSV data
     search      Search CSV data with a regex
@@ -80,6 +83,7 @@ macro_rules! command_list {
     sniff       Quickly sniff CSV metadata
     sort        Sort CSV data in alphabetical, numerical, reverse or random order
     sortcheck   Check if a CSV is sorted
+    sqlp        Run a SQL query against several CSVs using the Pola.rs engine
     stats       Infer data types and compute summary statistics
     validate    Validate CSV data for RFC4180-compliance or with JSON Schema
 
@@ -231,6 +235,7 @@ Please choose one of the following commands:",
 enum Command {
     ApplyDP,
     Count,
+    Datefmt,
     Dedup,
     Describegpt,
     Diff,
@@ -242,11 +247,14 @@ enum Command {
     Help,
     Index,
     Input,
+    #[cfg(feature = "polars")]
+    JoinP,
     #[cfg(feature = "luau")]
     Luau,
     Pseudo,
     Rename,
     Replace,
+    Reverse,
     Safenames,
     Sample,
     Search,
@@ -257,6 +265,8 @@ enum Command {
     Sniff,
     Sort,
     SortCheck,
+    #[cfg(feature = "polars")]
+    SqlP,
     Stats,
     Validate,
 }
@@ -278,6 +288,7 @@ impl Command {
         match self {
             Command::ApplyDP => cmd::applydp::run(argv),
             Command::Count => cmd::count::run(argv),
+            Command::Datefmt => cmd::datefmt::run(argv),
             Command::Dedup => cmd::dedup::run(argv),
             Command::Describegpt => cmd::describegpt::run(argv),
             Command::Diff => cmd::diff::run(argv),
@@ -293,11 +304,14 @@ impl Command {
             },
             Command::Index => cmd::index::run(argv),
             Command::Input => cmd::input::run(argv),
+            #[cfg(feature = "polars")]
+            Command::JoinP => cmd::joinp::run(argv),
             #[cfg(feature = "luau")]
             Command::Luau => cmd::luau::run(argv),
             Command::Pseudo => cmd::pseudo::run(argv),
             Command::Rename => cmd::rename::run(argv),
             Command::Replace => cmd::replace::run(argv),
+            Command::Reverse => cmd::reverse::run(argv),
             Command::Safenames => cmd::safenames::run(argv),
             Command::Sample => cmd::sample::run(argv),
             Command::Search => cmd::search::run(argv),
@@ -308,6 +322,8 @@ impl Command {
             Command::Sniff => cmd::sniff::run(argv),
             Command::Sort => cmd::sort::run(argv),
             Command::SortCheck => cmd::sortcheck::run(argv),
+            #[cfg(feature = "polars")]
+            Command::SqlP => cmd::sqlp::run(argv),
             Command::Stats => cmd::stats::run(argv),
             Command::Validate => cmd::validate::run(argv),
         }
