@@ -19,6 +19,27 @@ fn excel_open_xls() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn excel_cellerrors() {
+    let wrk = Workdir::new("excel_cellerrors");
+
+    let xls_file = wrk.load_test_file("excel-xlsx.xlsx");
+
+    let mut cmd = wrk.command("excel");
+    cmd.args(["--sheet", "cellerrors"]).arg(xls_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["col1", "col 2", "column-3"],
+        svec!["1", "-50", "15"],
+        svec!["2", "#DIV/0!", "#NAME?"],
+        svec!["3", "50", "20"],
+        svec!["4", "33.333333333333336", "3"],
+        svec!["5", "25", "4"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn excel_open_xls_delimiter() {
     let wrk = Workdir::new("excel_open_xls_delimiter");
@@ -52,6 +73,20 @@ fn excel_open_xlsx_readpassword() {
     wrk.assert_err(&mut cmd);
 }
 
+#[test]
+fn excel_open_ods_readpassword() {
+    let wrk = Workdir::new("excel_open_ods_readpassword");
+
+    let ods_file = wrk.load_test_file("password-protected-password123.ods");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg(ods_file);
+
+    let got = wrk.output_stderr(&mut cmd);
+    assert_eq!(got, "Ods error: Workbook is password protected\n");
+    wrk.assert_err(&mut cmd);
+}
+
 #[test]
 fn excel_open_flexible_xls() {
     let wrk = Workdir::new("excel_open_flexible_xls");
@@ -176,18 +211,18 @@ fn excel_date_xlsx_date_format() {
     let expected = vec![
         svec!["date", "plaincol"],
         svec![
-            "1980-12-25",
+            "Thu 1980-12-25",
             "it will still parse the dates below as date even if plaincol is not in the default \
              --dates-whitelist because the cell format was set to date"
         ],
-        svec!["Tue 2001-09-11 08:30:00", "2001-09-11"],
+        svec!["Tue 2001-09-11 08:30:00", "Tue 2001-09-11"],
         svec!["not a date", "Tue 2001-09-11 08:30:00"],
         svec![
             "Wednesday, Mar-14-2012",
             "the date below is not parsed as a date coz we didn't explicitly set the cell format \
              to a date format and \"plaincol\" is not in the --dates-whitelist"
         ],
-        svec!["2001-09-11", "9/11/01 8:30 am"],
+        svec!["Tue 2001-09-11", "9/11/01 8:30 am"],
     ];
     assert_eq!(got, expected);
 }
@@ -204,11 +239,11 @@ fn excel_xlsx_data_types() {
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["int", "float", "bool", "date", "duration", "string", "emojis", "foreign"], 
-        svec!["1", "1.1", "true", "2001-09-11", "0.4305555555555556", "The", "The", "敏捷的棕色狐狸在森林里奔跑"], 
-        svec!["2", "1.32434354545454", "false", "2023-10-07", "0.989849537037037", "quick", "🍔", "Franz jagt im komplett verwahrlosten Taxi quer durch Bayern"],
-        svec!["3", "0.423546456564534", "1", "1941-12-07", "1.2815162037037038", "brown", "is", "Le rusé goupil franchit d'un bond le chien somnolent."], 
-        svec!["4", "-54545.6565756785", "0", "2001-09-11 08:30:00", "0.9791666666666666", "fox", "💩", "El rápido zorro marrón"],
-        svec!["5", "-5446563454.43546", "true", "1945-08-06 08:15:00", "0.0004629629629629629", "jumped", "🙀", "いろはにほへとちりぬるをわかよたれそつねならむうゐのおくやまけふこえてあさきゆめみしゑひもせす"]
+        svec!["1", "1.1", "true", "2001-09-11", "PT37200S", "The", "The", "敏捷的棕色狐狸在森林里奔跑"], 
+        svec!["2", "1.32434354545454", "false", "2023-10-07", "PT85523S", "quick", "🍔", "Franz jagt im komplett verwahrlosten Taxi quer durch Bayern"], 
+        svec!["3", "0.423546456564534", "1", "1941-12-07", "PT110723S", "brown", "is", "Le rusé goupil franchit d'un bond le chien somnolent."], 
+        svec!["4", "-54545.6565756785", "0", "2001-09-11 08:30:00", "PT84600S", "fox", "💩", "El rápido zorro marrón"], 
+        svec!["5", "-5446563454.43546", "true", "1945-08-06 08:15:00", "PT40S", "jumped", "🙀", "いろはにほへとちりぬるをわかよたれそつねならむうゐのおくやまけふこえてあさきゆめみしゑひもせす"]
     ];
     assert_eq!(got, expected);
 }
@@ -565,6 +600,31 @@ fn excel_metadata() {
     wrk.assert_success(&mut cmd);
 }
 
+#[test]
+fn excel_short_metadata() {
+    let wrk = Workdir::new("excel_short_metadata");
+
+    let xls_file = wrk.load_test_file("excel-xls.xls");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--metadata").arg("short").arg(xls_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["index", "sheet_name", "type", "visible"],
+        svec!["0", "First", "WorkSheet", "Visible"],
+        svec!["1", "Flexibility Test", "WorkSheet", "Visible"],
+        svec!["2", "Middle", "WorkSheet", "Visible"],
+        svec!["3", "Sheet1", "WorkSheet", "Visible"],
+        svec!["4", "trim test", "WorkSheet", "Visible"],
+        svec!["5", "date test", "WorkSheet", "Visible"],
+        svec!["6", "NoData", "WorkSheet", "Visible"],
+        svec!["7", "Last", "WorkSheet", "Visible"],
+    ];
+    assert_eq!(got, expected);
+    wrk.assert_success(&mut cmd);
+}
+
 #[test]
 fn excel_metadata_pretty_json() {
     let wrk = Workdir::new("excel_metadata");
@@ -797,6 +857,25 @@ fn ods_metadata() {
     wrk.assert_success(&mut cmd);
 }
 
+#[test]
+fn ods_short_metadata() {
+    let wrk = Workdir::new("ods_short_metadata");
+
+    let xls_file = wrk.load_test_file("excel-ods.ods");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--metadata").arg("s").arg(xls_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["index", "sheet_name", "type", "visible"],
+        svec!["0", "Sheet1", "WorkSheet", "Visible"],
+    ];
+
+    assert_eq!(got, expected);
+    wrk.assert_success(&mut cmd);
+}
+
 #[test]
 fn ods_metadata_pretty_json() {
     let wrk = Workdir::new("ods_metadata_pretty_json");
