@@ -341,36 +341,39 @@ fn sqlp_null_aware_equality_checks() {
 }
 
 #[test]
-fn sqlp_rnull_values() {
-    let wrk = Workdir::new("sqlp_rnull_values");
+fn sqlp_rnull_values_wnull_value() {
+    let wrk = Workdir::new("sqlp_rnull_values_wnull_value");
     wrk.create(
         "test_null.csv",
         vec![
             svec!["a", "b"],
-            svec!["1", "NULL"],
+            svec!["1", "Nothing"],
             svec!["2", "NA"],
             svec!["3", "Dunno"],
             svec!["4", "4"],
             svec!["5", ""],
             svec!("6", "6"),
+            svec!("7", "DUNNO"),
         ],
     );
 
     let mut cmd = wrk.command("sqlp");
 
     cmd.arg("test_null.csv")
-        .args(["--rnull-values", "NULL,NA,Dunno"])
+        .args(["--rnull-values", "Nothing,NA,<empty string>,Dunno"])
+        .args(["--wnull-value", "NULL"])
         .arg("SELECT * FROM test_null");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["a", "b"],
-        svec!["1", ""],
-        svec!["2", ""],
-        svec!["3", ""],
+        svec!["1", "NULL"],
+        svec!["2", "NULL"],
+        svec!["3", "NULL"],
         svec!["4", "4"],
-        svec!["5", ""],
+        svec!["5", "NULL"],
         svec!["6", "6"],
+        svec!["7", "DUNNO"],
     ];
     assert_eq!(got, expected);
 }
@@ -491,7 +494,7 @@ fn sqlp_string_functions() {
         svec!["scol"],
         svec!["true"],
         svec!["true"],
-        svec!["true"],
+        svec!["false"],
         svec!["true"],
         svec!["false"],
     ];
@@ -523,7 +526,7 @@ fn sqlp_string_functions() {
         svec!["scol"],
         svec!["abc"],
         svec!["abc"],
-        svec!["abc"],
+        svec!["   "],
         svec!["a"],
         svec!["b"],
     ];
@@ -539,7 +542,7 @@ fn sqlp_string_functions() {
         svec!["scol"],
         svec!["bc"],
         svec!["bc"],
-        svec!["bc"],
+        svec!["  "],
         svec![""],
         svec![""],
     ];
@@ -555,7 +558,7 @@ fn sqlp_string_functions() {
         svec!["scol"],
         svec!["ABCDE"],
         svec!["ABC"],
-        svec!["ABC"],
+        svec!["    ABC"],
         svec!["A"],
         svec!["B"],
     ];
@@ -571,7 +574,7 @@ fn sqlp_string_functions() {
         svec!["scol"],
         svec!["abcde"],
         svec!["abc"],
-        svec!["abc"],
+        svec!["    abc"],
         svec!["a"],
         svec!["b"],
     ];
@@ -587,7 +590,7 @@ fn sqlp_string_functions() {
         svec!["scol"],
         svec!["5"],
         svec!["3"],
-        svec!["3"],
+        svec!["7"],
         svec!["1"],
         svec!["1"],
     ];
@@ -603,7 +606,7 @@ fn sqlp_string_functions() {
         svec!["scol"],
         svec!["5"],
         svec!["3"],
-        svec!["3"],
+        svec!["7"],
         svec!["1"],
         svec!["1"],
     ];
@@ -865,6 +868,29 @@ select ward,count(*) as cnt from temp_table2 group by ward order by cnt desc, wa
         .arg("test.sql")
         .args(["--format", "json"]);
 
+    let got: String = wrk.stdout(&mut cmd);
+    let expected = r#"[{"ward":"Ward 3","cnt":2},{"ward":" ","cnt":1},{"ward":"04","cnt":1},{"ward":"3","cnt":1},{"ward":"Ward 13","cnt":1},{"ward":"Ward 17","cnt":1},{"ward":"Ward 19","cnt":1},{"ward":"Ward 21","cnt":1},{"ward":"Ward 6","cnt":1}]"#;
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_boston311_sql_script_jsonl() {
+    let wrk = Workdir::new("sqlp_boston311_sql_script_jsonl");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    wrk.create_from_string(
+        "test.sql",
+        r#"create table temp_table as select * from "boston311-100" where ontime = 'OVERDUE';
+create table temp_table2 as select * from temp_table limit 10;
+select ward,count(*) as cnt from temp_table2 group by ward order by cnt desc, ward asc;"#,
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg(&test_file)
+        .arg("test.sql")
+        .args(["--format", "jsonl"]);
+
     let got: String = wrk.stdout(&mut cmd);
     let expected = r#"{"ward":"Ward 3","cnt":2}
 {"ward":" ","cnt":1}
@@ -1180,3 +1206,711 @@ fn sqlp_sql_tsv() {
 
     assert_eq!(got, expected);
 }
+
+#[test]
+fn sqlp_binary_functions() {
+    let wrk = Workdir::new("sqlp_sql_binary_functions");
+    wrk.create("dummy.csv", vec![svec!["dummy"], svec!["0"]]);
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("dummy.csv")
+        .arg(
+            r#"
+        SELECT *,
+          -- bit strings
+          b''                 AS b0,
+          b'1001'             AS b1,
+          b'11101011'         AS b2,
+          b'1111110100110010' AS b3,
+          -- hex strings
+          x''                 AS x0,
+          x'FF'               AS x1,
+          x'4142'             AS x2,
+          x'DeadBeef'         AS x3,
+        FROM dummy
+        "#,
+        )
+        .args(["--format", "parquet"]);
+
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn sqlp_length_fns() {
+    let wrk = Workdir::new("sqlp_sql_length_fns");
+    wrk.create(
+        "test.csv",
+        vec![svec!["words"], svec!["Cafe"], svec![""], svec!["東京"]],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg(
+        r#"
+        SELECT
+              words,
+              LENGTH(words) AS n_chrs1,
+              CHAR_LENGTH(words) AS n_chrs2,
+              CHARACTER_LENGTH(words) AS n_chrs3,
+              OCTET_LENGTH(words) AS n_bytes,
+              BIT_LENGTH(words) AS n_bits
+            FROM test
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["words", "n_chrs1", "n_chrs2", "n_chrs3", "n_bytes", "n_bits"],
+        svec!["Cafe", "4", "4", "4", "4", "32"],
+        svec!["", "0", "0", "0", "0", "0"],
+        svec!["東京", "2", "2", "2", "6", "48"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_control_flow() {
+    let wrk = Workdir::new("sqlp_control_flow");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["x", "y", "z"],
+            svec!["1", "5", "3"],
+            svec!["", "4", "4"],
+            svec!["2", "", ""],
+            svec!["3", "3", "3"],
+            svec!["", "", "6"],
+            svec!["4", "2", ""],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg(
+        r#"
+        SELECT
+          COALESCE(x,y,z) as "coalsc",
+          NULLIF(x, y) as "nullif x_y",
+          NULLIF(y, z) as "nullif y_z",
+          IFNULL(x, y) as "ifnull x_y",
+          IFNULL(y,-1) as "inullf y_z",
+          COALESCE(x, NULLIF(y,z)) as "both",
+          IF(x = y, 'eq', 'ne') as "x_eq_y",
+        FROM test
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "coalsc",
+            "nullif x_y",
+            "nullif y_z",
+            "ifnull x_y",
+            "inullf y_z",
+            "both",
+            "x_eq_y"
+        ],
+        svec!["1", "1", "5", "1", "5", "1", "ne"],
+        svec!["4", "", "", "4", "4", "", "ne"],
+        svec!["2", "2", "", "2", "-1", "2", "ne"],
+        svec!["3", "", "", "3", "3", "3", "eq"],
+        svec!["6", "", "", "", "-1", "", "ne"],
+        svec!["4", "4", "2", "4", "2", "4", "ne"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_div_sign() {
+    let wrk = Workdir::new("sqlp_div_sign");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["a", "b"],
+            svec!["10.0", "-100.5"],
+            svec!["20.0", "7.0"],
+            svec!["30.0", "2.5"],
+            svec!["40.0", ""],
+            svec!["50.0", "-3.14"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg(
+        r#"
+        SELECT
+            a / b AS a_div_b,
+            a // b AS a_floordiv_b,
+            SIGN(b) AS b_sign,
+        FROM test
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["a_div_b", "a_floordiv_b", "b_sign"],
+        svec!["-0.09950248756218906", "-1", "-1"],
+        svec!["2.857142857142857", "2", "1"],
+        svec!["12.0", "12", "1"],
+        svec!["", "", ""],
+        svec!["-15.92356687898089", "-16", "-1"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_string_replace() {
+    let wrk = Workdir::new("sqlp_string_replace");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["words"],
+            svec!["Yemeni coffee is the best coffee"],
+            svec![""],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg(
+        r#"
+        SELECT
+        REPLACE(
+          REPLACE(words, 'coffee', 'tea'),
+          'Yemeni',
+          'English breakfast'
+        )
+        FROM test
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["words"],
+        svec!["English breakfast tea is the best tea"],
+        svec!(""),
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_compound_join_basic() {
+    let wrk = Workdir::new("sqlp_compound_join_basic");
+    wrk.create(
+        "test1.csv",
+        vec![
+            svec!["a", "b"],
+            svec!["1", "1"],
+            svec!["2", "3"],
+            svec!["3", "4"],
+            svec!["4", "4"],
+            svec!["5", "5"],
+        ],
+    );
+
+    wrk.create(
+        "test2.csv",
+        vec![
+            svec!["a", "b"],
+            svec!["1", "0"],
+            svec!["2", "3"],
+            svec!["3", "4"],
+            svec!["4", "5"],
+            svec!["5", "6"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.args(["test1.csv", "test2.csv"])
+        .arg(
+            r#"
+        SELECT * FROM test1
+         INNER JOIN test2 ON test1.a = test2.a AND test1.b = test2.b
+"#,
+        )
+        .arg("--truncate-ragged-lines");
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![svec!["a", "b",], svec!["2", "3"], svec!["3", "4"]];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_compound_join_diff_colnames() {
+    let wrk = Workdir::new("sqlp_compound_join_diff_colnames");
+    wrk.create(
+        "test1.csv",
+        vec![
+            svec!["a", "b"],
+            svec!["1", "1"],
+            svec!["2", "2"],
+            svec!["3", "3"],
+            svec!["4", "4"],
+            svec!["5", "5"],
+        ],
+    );
+
+    wrk.create(
+        "test2.csv",
+        vec![
+            svec!["a", "b", "c"],
+            svec!["0", "1", "7"],
+            svec!["2", "2", "8"],
+            svec!["3", "3", "9"],
+            svec!["4", "5", "10"],
+            svec!["5", "6", "11"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.args(["test1.csv", "test2.csv"])
+        .arg(
+            r#"
+        SELECT * FROM test1
+         INNER JOIN test2 ON test1.a = test2.b AND test1.b = test2.a
+"#,
+        )
+        .arg("--truncate-ragged-lines");
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["a", "b", "c"],
+        svec!["2", "2", "8"],
+        svec!["3", "3", "9"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_compound_join_three_tables() {
+    let wrk = Workdir::new("sqlp_compound_join_three_tables");
+    wrk.create(
+        "test1.csv",
+        vec![
+            svec!["a", "b"],
+            svec!["1", "1"],
+            svec!["2", "3"],
+            svec!["3", "4"],
+            svec!["4", "4"],
+            svec!["5", "5"],
+        ],
+    );
+
+    wrk.create(
+        "test2.csv",
+        vec![
+            svec!["a", "b"],
+            svec!["1", "0"],
+            svec!["2", "3"],
+            svec!["3", "4"],
+            svec!["4", "5"],
+            svec!["5", "6"],
+        ],
+    );
+
+    wrk.create(
+        "test3.csv",
+        vec![
+            svec!["a", "b", "c"],
+            svec!["1", "0", "0"],
+            svec!["2", "3", "3"],
+            svec!["3", "4", "4"],
+            svec!["4", "5", "5"],
+            svec!["5", "6", "6"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.args(["test1.csv", "test2.csv", "test3.csv"])
+        .arg(
+            r#"
+            SELECT * FROM test1
+            INNER JOIN test2
+                ON test1.a = test2.a AND test1.b = test2.b
+            INNER JOIN test3
+                ON test1.a = test3.a AND test1.b = test3.b
+"#,
+        )
+        .arg("--truncate-ragged-lines");
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["a", "b", "c"],
+        svec!["2", "3", "3"],
+        svec!["3", "4", "4"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_string_concat() {
+    let wrk = Workdir::new("sqlp_string_concat");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["x", "y", "z"],
+            svec!["a", "d", "1"],
+            svec!["", "e", "2"],
+            svec!["c", "f", "3"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg(
+        r#"
+        SELECT
+           ("x" || "x" || "y")           AS c0,
+           ("x" || "y" || "z")           AS c1,
+           CONCAT(("x" + "x"), "y")      AS c2,
+           CONCAT("x", "x", "y")         AS c3,
+           CONCAT("x", "y", ("z" * 2))   AS c4,
+           CONCAT_WS(':', "x", "y", "z") AS c5,
+           CONCAT_WS('!', "x", "y", "z") AS c6,
+        FROM test
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["c0", "c1", "c2", "c3", "c4", "c5", "c6"],
+        svec!["aad", "ad1", "aad", "aad", "ad2", "a:d:1", "a!d!1"],
+        svec!["", "", "e", "e", "e4", "e:2", "e!2"],
+        svec!["ccf", "cf3", "ccf", "ccf", "cf6", "c:f:3", "c!f!3"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_string_right_reverse() {
+    let wrk = Workdir::new("sqlp_string_right_reverse");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["txt"],
+            svec!["abcde"],
+            svec!["abc"],
+            svec!["a"],
+            svec![""],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg(
+        r#"
+        SELECT
+           LEFT(txt,2) AS "l",
+           RIGHT(txt,2) AS "r",
+           REVERSE(txt) AS "rev"
+        FROM test
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["l", "r", "rev"],
+        svec!["ab", "de", "edcba"],
+        svec!["ab", "bc", "cba"],
+        svec!["a", "a", "a"],
+        svec!["", "", ""],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_modulo() {
+    let wrk = Workdir::new("sqlp_modulo");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["a", "b", "c", "d"],
+            svec!["1.5", "6", "11", "16.5"],
+            svec!["", "7", "12", "17.0"],
+            svec!["3.0", "8", "13", "18.5"],
+            svec!["4,333333333", "9", "14", ""],
+            svec!["5.0", "10", "15", "20.0"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg(
+        r#"
+        SELECT
+            cast(a as float) % 2 AS a2,
+            cast(b as float)  % 3 AS b3,
+            MOD(cast(c as float), 4) AS c4,
+            MOD(cast(d as float), 5.5) AS d55
+        FROM test
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["a2", "b3", "c4", "d55"],
+        svec!["1.5", "0.0", "3.0", "0.0"],
+        svec!["", "1.0", "0.0", "0.5"],
+        svec!["1.0", "2.0", "1.0", "2.0"],
+        svec!["", "0.0", "2.0", ""],
+        svec!["1.0", "1.0", "3.0", "3.5"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_stddev_variance() {
+    let wrk = Workdir::new("sqlp_stddev_variance");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["v1", "v2", "v3", "v4"],
+            svec!["-1.0", "5.5", "-10", "-100"],
+            svec!["0.0", "0.0", "", "0.0"],
+            svec!["1.0", "3.0", "10", "-50.0"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg(
+        r#"
+        SELECT
+            STDEV(v1) AS "v1_std",
+            STDDEV(v2) AS "v2_std",
+            STDEV_SAMP(v3) AS "v3_std",
+            STDDEV_SAMP(v4) AS "v4_std",
+            VAR(v1) AS "v1_var",
+            VARIANCE(v2) AS "v2_var",
+            VARIANCE(v3) AS "v3_var",
+            VAR_SAMP(v4) AS "v4_var"
+        FROM test
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["v1_std", "v2_std", "v3_std", "v4_std", "v1_var", "v2_var", "v3_var", "v4_var"],
+        svec![
+            "1.0",
+            "2.753785273643051",
+            "14.142135623730951",
+            "50.0",
+            "1.0",
+            "7.583333333333334",
+            "200.0",
+            "2500.0"
+        ],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_string_position() {
+    let wrk = Workdir::new("sqlp_string_position");
+    wrk.create(
+        "cities.csv",
+        vec![
+            svec!["city"],
+            svec!["Dubai"],
+            svec!["Abu Dhabi"],
+            svec!["Sharjah"],
+            svec!["Al Ain"],
+            svec!["Ajman"],
+            svec!["Ras Al Khaimah"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("cities.csv").arg(
+        r#"
+        SELECT
+        POSITION('a' IN city) AS a_lc1,
+        POSITION('A' IN city) AS a_uc1,
+        STRPOS(city,'a') AS a_lc2,
+        STRPOS(city,'A') AS a_uc2,
+      FROM cities
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["a_lc1", "a_uc1", "a_lc2", "a_uc2"],
+        svec!["4", "0", "4", "0"],
+        svec!["7", "1", "7", "1"],
+        svec!["3", "0", "3", "0"],
+        svec!["0", "1", "0", "1"],
+        svec!["4", "1", "4", "1"],
+        svec!["2", "5", "2", "5"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_date_part() {
+    let wrk = Workdir::new("sqlp_date_part");
+    wrk.create(
+        "datestbl.csv",
+        vec![svec!["datecol"], svec!["20-01-2012 10:30:20"]],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("datestbl.csv")
+        .arg(
+            r#"
+        SELECT 
+            DATE_PART(datecol, 'isoyear') as c1,
+            DATE_PART(datecol, 'month') as c2,
+            DATE_PART(datecol, 'day') as c3,
+            DATE_PART(datecol, 'hour') as c4,
+            DATE_PART(datecol, 'minute') as c5,
+            DATE_PART(datecol, 'second') as c6,
+            DATE_PART(datecol, 'millisecond') as c61,
+            DATE_PART(datecol, 'microsecond') as c62,
+            DATE_PART(datecol, 'nanosecond') as c63,
+            DATE_PART(datecol, 'isoweek') as c7,
+            DATE_PART(datecol, 'dayofyear') as c8,
+            DATE_PART(datecol, 'dayofweek') as c9,
+            DATE_PART(datecol, 'time') as c10,
+            DATE_PART(datecol, 'decade') as c11,
+            DATE_PART(datecol, 'century') as c12,
+            DATE_PART(datecol, 'millennium') as c13,
+            DATE_PART(datecol, 'quarter') as c14,
+      FROM datestbl
+"#,
+        )
+        .arg("--try-parsedates");
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "c1", "c2", "c3", "c4", "c5", "c6", "c61", "c62", "c63", "c7", "c8", "c9", "c10",
+            "c11", "c12", "c13", "c14"
+        ],
+        svec![
+            "2012",
+            "1",
+            "20",
+            "10",
+            "30",
+            "20",
+            "20000.0",
+            "20000000.0",
+            "20000000000.0",
+            "3",
+            "20",
+            "5",
+            "10:30:20.000000000",
+            "201",
+            "21",
+            "3",
+            "1"
+        ],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_date() {
+    let wrk = Workdir::new("sqlp_date");
+    wrk.create("dummy.csv", vec![svec!["c1"], svec![""]]);
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("dummy.csv").arg(
+        r#"
+        SELECT 
+            DATE('2021-03-15') as c1,
+            DATE('2021-03-15 10:30:20', '%Y-%m-%d %H:%M:%S') as c2,
+        FROM dummy
+"#,
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![svec!["c1", "c2"], svec!["2021-03-15", "2021-03-15"]];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_autotab_delim() {
+    let wrk = Workdir::new("sqlp_autotab_delim");
+    wrk.create_with_delim(
+        "cities_array.tsv",
+        vec![
+            svec!["countryid", "cities"],
+            svec!["DB", "Dubai,Abu Dhabi,Sharjah"],
+            svec!["IN", "Mumbai,Delhi,Bangalore"],
+            svec!["US", "New York,Los Angeles,Chicago"],
+            svec!["UK", "London,Birmingham,Manchester"],
+            svec!["CN", "Beijing"],
+            svec!["RU", "Moscow"],
+            svec!["NL", "Amsterdam"],
+            svec!["IT", "Rome,Milan,Turin,Naples,Venice"],
+        ],
+        b'\t',
+    );
+
+    let output_file = wrk.path("output.tsv").to_string_lossy().to_string();
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("cities_array.tsv")
+        .arg(
+            r#"
+            SELECT 
+                countryid, cities
+            FROM cities_array
+    "#,
+        )
+        .args(["--output", &output_file]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got = wrk.read_to_string(&output_file);
+    let expected = r#"countryid	cities
+DB	Dubai,Abu Dhabi,Sharjah
+IN	Mumbai,Delhi,Bangalore
+US	New York,Los Angeles,Chicago
+UK	London,Birmingham,Manchester
+CN	Beijing
+RU	Moscow
+NL	Amsterdam
+IT	Rome,Milan,Turin,Naples,Venice
+"#;
+
+    assert_eq!(got, expected);
+}
